"""Tests for missing-return-doc and missing-return-type-doc for Sphinx style docstrings
with accept-no-returns-doc = no"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument, disallowed-name, too-few-public-methods
# pylint: disable=line-too-long


def my_func(self):  # [missing-return-type-doc]
    """Warn partial sphinx returns

    :returns: Always False
    """
    return False


def my_func(self) -> bool:
    """Sphinx missing return type with annotations

    :returns: Always False
    """
    return False


def my_func(self):  # [missing-return-doc]
    """Warn partial sphinx returns type

    :rtype: bool
    """
    return False


def warn_missing_sphinx_returns(  # [missing-return-type-doc, missing-return-doc]
    self, doc_type
):
    """This is a docstring.

    :param doc_type: Sphinx
    :type doc_type: str
    """
    return False


def my_func(self):  # [missing-return-doc]
    """warns_sphinx_return_list_of_custom_class_without_description

    :rtype: list(:class:`mymodule.Class`)
    """
    return [mymodule.Class()]


class Foo:
    """test_finds_missing_property_return_type_sphinx
    Example of a property having missing return documentation in
    a Sphinx style docstring
    """

    @property
    def foo(self):  # [missing-return-type-doc]
        """docstring ...

        :raises RuntimeError: Always
        """
        raise RuntimeError()
        return 10  # [unreachable]


class Foo:
    """Example of a class function trying to use `type` as return
    documentation in a Sphinx style docstring
    """

    def test_ignores_non_property_return_type_sphinx(  # [missing-return-doc, missing-return-type-doc]
        self,
    ):
        """docstring ...

        :type: int
        """
        return 10



# pylint: disable=missing-function-docstring

"""
Regression tests for StopIteration raised when using limit-inference-results=0
"""


def get_youtube_link(song_name: str, song_artists):
    with open("possible errors.txt", "ab") as file:
        file.write(f"{', '.join(song_artists)} - {song_name}".encode())



"""Regression tests for https://github.com/pylint-dev/pylint/issues/4660"""

# pylint: disable=useless-return, unused-argument
# pylint: disable=missing-docstring, too-few-public-methods, invalid-name

from __future__ import annotations

from typing import Union, Any, Literal, overload
from collections.abc import Callable


def my_print(*args: Any) -> None:
    print(", ".join(str(x) for x in args))
    return


# ---- This is OK ----
class MyClass:
    def my_method(self, option: Literal["mandatory"]) -> Callable[..., Any]:
        return my_print


c = MyClass().my_method("mandatory")
c(1, "foo")

# ---- This runs OK but pylint reports an error ----
class MyClass1:
    @overload
    def my_method(self, option: Literal["mandatory"]) -> Callable[..., Any]:
        ...

    @overload
    def my_method(
        self, option: Literal["optional", "mandatory"]
    ) -> Union[None, Callable[..., Any]]:
        ...

    def my_method(
        self, option: Literal["optional", "mandatory"]
    ) -> Union[None, Callable[..., Any]]:
        return my_print


d = MyClass1().my_method("mandatory")
d(1, "bar")  # [not-callable]



"""Tests for annotation of variables and potential use before assignment"""
# pylint: disable=too-few-public-methods, global-variable-not-assigned
from collections import namedtuple
from typing import List

def value_and_type_assignment():
    """The variable assigned a value and type"""
    variable: int = 2
    print(variable)


def only_type_assignment():
    """The variable never gets assigned a value"""
    variable: int
    print(variable)  # [used-before-assignment]


def both_type_and_value_assignment():
    """The variable first gets a type and subsequently a value"""
    variable: int
    variable = 1
    print(variable)


def value_assignment_after_access():
    """The variable gets a value after it has been accessed"""
    variable: int
    print(variable)  # [used-before-assignment]
    variable = 1


def value_assignment_from_iterator():
    """The variables gets a value from an iterator"""
    variable: int
    for variable in (1, 2):
        print(variable)


def assignment_in_comprehension():
    """A previously typed variables gets used in a comprehension. Don't crash!"""
    some_list: List[int]
    some_list = [1, 2, 3]
    some_list = [i * 2 for i in some_list]


def decorator_returning_function():
    """A decorator that returns a wrapper function with decoupled typing"""
    def wrapper_with_decoupled_typing():
        print(var)

    var: int
    var = 2
    return wrapper_with_decoupled_typing


def decorator_returning_incorrect_function():
    """A decorator that returns a wrapper function with decoupled typing"""
    def wrapper_with_type_and_no_value():
        # This emits NameError rather than UnboundLocalError, so
        # undefined-variable is okay, even though the traceback refers
        # to "free variable 'var' referenced before assignment"
        print(var) # [undefined-variable]

    var: int
    return wrapper_with_type_and_no_value


def typing_and_value_assignment_with_tuple_assignment():
    """The typed variables get assigned with a tuple assignment"""
    var_one: int
    var_two: int
    var_one, var_two = 1, 1
    print(var_one)
    print(var_two)


def nested_class_as_return_annotation():
    """A namedtuple as a class attribute is used as a return annotation

    Taken from https://github.com/pylint-dev/pylint/issues/5568"""
    class MyObject:
        """namedtuple as class attribute"""
        Coords = namedtuple('Point', ['x', 'y'])

        def my_method(self) -> Coords:
            """Return annotation is valid"""
            # pylint: disable=unnecessary-pass
            pass

    print(MyObject)



"""Since Python version 3.8, a class decorated with typing.final cannot be
subclassed."""

# pylint: disable=missing-docstring, too-few-public-methods

# Disabled because of a bug with pypy 3.8 see
# https://github.com/pylint-dev/pylint/pull/7918#issuecomment-1352737369
# pylint: disable=multiple-statements

from typing import final


@final
class Base:
    pass


class Subclass(Base):  # [subclassed-final-class]
    pass



"""Crash regression in astroid on Compare node inference
Fixed in https://github.com/pylint-dev/astroid/pull/1185"""
# pylint: disable=missing-docstring, broad-exception-raised


# Reported at https://github.com/pylint-dev/pylint/issues/5048
def func(parameter):
    if tuple() + (parameter[1],) in set():
        raise Exception()



"""invalid ascii char in a for loop"""
# pylint: disable=consider-using-join
import os


def main():
    """main func"""
    # +2: [non-ascii-name]
    a_variable = ""
    for łol in os.listdir("."):
        # Usage should not raise a second error
        a_variable += (
            f"{łol}                                                                       "
        )



# pylint: disable=missing-docstring,pointless-statement,unnecessary-comprehension

var = {1, 2, 3}

for x in var:
    pass
for x in {1, 2, 3}:  # [use-sequence-for-iteration]
    pass

(x for x in var)
(x for x in {1, 2, 3})  # [use-sequence-for-iteration]

[x for x in var]
[x for x in {1, 2, 3}]  # [use-sequence-for-iteration]

[x for x in {*var, 4}]

def deduplicate(list_in):
    for thing in {*list_in}:
        print(thing)

def deduplicate_two_lists(input1, input2):
    for thing in {*input1, *input2}:
        print(thing)

def deduplicate_nested_sets(input1, input2, input3, input4):
    for thing in {{*input1, *input2}, {*input3, *input4}}:
        print(thing)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

import astroid

from pylint.checkers import design_analysis
from pylint.testutils import CheckerTestCase, set_config


class TestDesignChecker(CheckerTestCase):
    CHECKER_CLASS = design_analysis.MisdesignChecker

    @set_config(
        ignored_parents=(".Dddd",),
        max_parents=1,
    )
    def test_too_many_ancestors_ignored_parents_are_skipped(self) -> None:
        """Make sure that classes listed in ``ignored-parents`` aren't counted
        by the too-many-ancestors message.
        """

        node = astroid.extract_node(
            """
        class Aaaa(object):
            pass
        class Bbbb(Aaaa):
            pass
        class Cccc(Bbbb):
            pass
        class Dddd(Cccc):
            pass
        class Eeee(Dddd):
            pass
        """
        )
        with self.assertNoMessages():
            self.checker.visit_classdef(node)

    @set_config(exclude_too_few_public_methods="toml.*")
    def test_exclude_too_few_methods_with_value(self) -> None:
        """Test exclude-too-few-public-methods option with value."""
        options = self.linter.config.exclude_too_few_public_methods

        assert any(i.match("toml") for i in options)
        assert any(i.match("toml.*") for i in options)
        assert any(i.match("toml.TomlEncoder") for i in options)

    def test_ignore_paths_with_no_value(self) -> None:
        """Test exclude-too-few-public-methods option with no value.
        Compare against actual list to see if validator works.
        """
        options = self.linter.config.exclude_too_few_public_methods

        assert options == []



"""Tests for potential-index-error"""
# pylint: disable=invalid-name, undefined-variable, missing-function-docstring, unused-variable

# Check assignment on same line
index = 3
print([1, 2, 3][3])  # [potential-index-error]
print((1, 2, 3)[3])  # [potential-index-error]
print([1, 2, 3][index])  # [potential-index-error]

# Check assignment on previous line
# We (currently) do not raise here to avoid risks of false positives
a_list = [1, 2, 3]
print(a_list[3])

# Test for uninferable lists
def my_func():
    an_inner_list = [1, 2, 3]


print(an_inner_list[3])


# Test that we don't crash on more complicated indices/slices
# We do not raise here (currently)
print([1, 2, 3][2:3])



# pylint: disable=missing-module-docstring
# pylint: disable=too-few-public-methods
import copy

class MyData:
    '''
    class docstring
    '''
    def __init__(self):
        self.data = {}

    def process(self):
        '''
        another method is responsible for putting "static_key"
        '''
        copy.copy(self.data['static_key'])



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import pytest

from pylint.__pkginfo__ import get_numversion_from_version


@pytest.mark.parametrize(
    "version,expected_numversion",
    [
        ["2.8.1", (2, 8, 1)],
        ["2.8.2+", (2, 8, 2)],
        ["3.0.0a0", (3, 0, 0)],
        ["3..0", (3, 0, 0)],
        ["1.a", (1, 0, 0)],
        ["", (0, 0, 0)],
        ["3.0.0b1", (3, 0, 0)],
        ["3.0.0rc1", (3, 0, 0)],
        ["3.0.0dev-234324234234f23abc4", (3, 0, 0)],
        ["pylint-2.4.7", (2, 4, 7)],
        ["2.8.3.dev3+g28c093c2.d20210428", (2, 8, 3)],
    ],
)
def test_numversion(version: str, expected_numversion: tuple[int, int, int]) -> None:
    assert get_numversion_from_version(version) == expected_numversion



"""Test generic alias support for typing.py types."""
# flake8: noqa
# pylint: disable=missing-docstring,pointless-statement,invalid-name
# pylint: disable=too-few-public-methods,multiple-statements,line-too-long, unnecessary-dunder-call
import abc
import typing

# special
typing.Tuple[int, int]
typing.Type[int]
typing.Callable[[int], str]

# builtins
typing.Dict[int, str]
typing.List[int]
typing.Set[int]
typing.FrozenSet[int]

# collections
typing.DefaultDict[int, str]
typing.OrderedDict[int, str]
typing.ChainMap[int, str]
typing.Counter[int]
typing.Deque[int]

# collections.abc
typing.AbstractSet[int]
typing.Collection[int]
typing.Container[int]
typing.ItemsView[int, str]
typing.KeysView[int]
typing.Mapping[int, str]
typing.MappingView[int]
typing.MutableMapping[int, str]
typing.MutableSequence[int]
typing.MutableSet[int]
typing.Sequence[int]
typing.ValuesView[int]

typing.Iterable[int]
typing.Iterator[int]
typing.Generator[int, None, None]
typing.Reversible[int]

typing.Coroutine[typing.List[str], str, int]
typing.AsyncGenerator[int, None]
typing.AsyncIterable[int]
typing.AsyncIterator[int]
typing.Awaitable[int]

# contextlib
typing.ContextManager[int]
typing.AsyncContextManager[int]

# re
typing.Pattern[str]
typing.Match[str]
typing.re.Pattern[str]
typing.re.Match[str]


# unsubscriptable types
typing.ByteString
typing.Hashable
typing.Sized
typing.ByteString[int]  # [unsubscriptable-object]
typing.Hashable[int]  # [unsubscriptable-object]
typing.Sized[int]  # [unsubscriptable-object]


# Missing implementation for 'collections.abc' derived classes
class DerivedHashable(typing.Hashable):  # [abstract-method]  # __hash__
    pass

class DerivedIterable(typing.Iterable[int]):  # [abstract-method]  # __iter__
    pass

class DerivedCollection(typing.Collection[int]):  # [abstract-method,abstract-method,abstract-method]  # __contains__, __iter__, __len__
    pass


# No implementation required for 'builtins' and 'collections' types
class DerivedList(typing.List[int]):
    def func(self):
        return self.__iter__()

class DerivedSet(typing.Set[int]):
    def func(self):
        return self.add(2)

class DerivedOrderedDict(typing.OrderedDict[int, str]):
    def func(self):
        return self.items()

class DerivedListIterable(typing.List[typing.Iterable[int]]):
    pass


# Multiple generic base classes
class DerivedMultiple(typing.Sized, typing.Hashable):  # [abstract-method,abstract-method]
    pass

class CustomAbstractCls1(abc.ABC):
    pass
class CustomAbstractCls2(typing.Sized, typing.Iterable[CustomAbstractCls1]):  # [abstract-method,abstract-method]  # __iter__, __len__
    pass
class CustomImplementation(CustomAbstractCls2):  # [abstract-method,abstract-method]  # __iter__, __len__
    pass


# Inheritance without generic
class DerivedList2(typing.List):
    pass

class DerivedOrderedDict2(typing.OrderedDict):
    pass

class DerivedIterable2(typing.Iterable):  # [abstract-method]  # __iter__
    pass


# Type annotations
var_tuple: typing.Tuple[int, int]
var_dict: typing.Dict[int, str]
var_orderedDict: typing.OrderedDict[int, str]
var_container: typing.Container[int]
var_sequence: typing.Sequence[int]
var_iterable: typing.Iterable[int]
var_awaitable: typing.Awaitable[int]
var_contextmanager: typing.ContextManager[int]
var_pattern: typing.Pattern[int]
var_pattern2: typing.re.Pattern[int]
var_bytestring: typing.ByteString
var_hashable: typing.Hashable
var_sized: typing.Sized

# Type annotation with unsubscriptable type
var_int: int[int]  # [unsubscriptable-object]
var_bytestring2: typing.ByteString[int]  # [unsubscriptable-object]
var_hashable2: typing.Hashable[int]  # [unsubscriptable-object]
var_sized2: typing.Sized[int]  # [unsubscriptable-object]


# Generic in type stubs only -> string annotations aren't checked
class A:
    ...

var_a1: A[str]  # [unsubscriptable-object]
var_a2: "A[str]"  # string annotations aren't checked
class B(A[str]):  # [unsubscriptable-object]
    ...



"""Tests for missing-timeout."""

# pylint: disable=consider-using-with,import-error,no-member,no-name-in-module,reimported

import requests
from requests import (
    delete,
    delete as delete_r,
    get,
    get as get_r,
    head,
    head as head_r,
    options,
    options as options_r,
    patch,
    patch as patch_r,
    post,
    post as post_r,
    put,
    put as put_r,
    request,
    request as request_r,
)

# requests without timeout
requests.delete("http://localhost")  # [missing-timeout]
requests.get("http://localhost")  # [missing-timeout]
requests.head("http://localhost")  # [missing-timeout]
requests.options("http://localhost")  # [missing-timeout]
requests.patch("http://localhost")  # [missing-timeout]
requests.post("http://localhost")  # [missing-timeout]
requests.put("http://localhost")  # [missing-timeout]
requests.request("call", "http://localhost")  # [missing-timeout]

delete_r("http://localhost")  # [missing-timeout]
get_r("http://localhost")  # [missing-timeout]
head_r("http://localhost")  # [missing-timeout]
options_r("http://localhost")  # [missing-timeout]
patch_r("http://localhost")  # [missing-timeout]
post_r("http://localhost")  # [missing-timeout]
put_r("http://localhost")  # [missing-timeout]
request_r("call", "http://localhost")  # [missing-timeout]

delete("http://localhost")  # [missing-timeout]
get("http://localhost")  # [missing-timeout]
head("http://localhost")  # [missing-timeout]
options("http://localhost")  # [missing-timeout]
patch("http://localhost")  # [missing-timeout]
post("http://localhost")  # [missing-timeout]
put("http://localhost")  # [missing-timeout]
request("call", "http://localhost")  # [missing-timeout]

kwargs_wo_timeout = {}
post("http://localhost", **kwargs_wo_timeout)  # [missing-timeout]

# requests valid cases
requests.delete("http://localhost", timeout=10)
requests.get("http://localhost", timeout=10)
requests.head("http://localhost", timeout=10)
requests.options("http://localhost", timeout=10)
requests.patch("http://localhost", timeout=10)
requests.post("http://localhost", timeout=10)
requests.put("http://localhost", timeout=10)
requests.request("call", "http://localhost", timeout=10)

delete_r("http://localhost", timeout=10)
get_r("http://localhost", timeout=10)
head_r("http://localhost", timeout=10)
options_r("http://localhost", timeout=10)
patch_r("http://localhost", timeout=10)
post_r("http://localhost", timeout=10)
put_r("http://localhost", timeout=10)
request_r("call", "http://localhost", timeout=10)

delete("http://localhost", timeout=10)
get("http://localhost", timeout=10)
head("http://localhost", timeout=10)
options("http://localhost", timeout=10)
patch("http://localhost", timeout=10)
post("http://localhost", timeout=10)
put("http://localhost", timeout=10)
request("call", "http://localhost", timeout=10)

kwargs_timeout = {'timeout': 10}
post("http://localhost", **kwargs_timeout)



"""Test that `yield` or `yield from` can't be used inside an async function."""
# pylint: disable=missing-docstring, unused-variable

async def good():
    def _inner():
        yield 42
        yield from [1, 2, 3]

async def good_two():
    # Starting from python 3.6 it's possible to yield inside async
    # https://www.python.org/dev/peps/pep-0525/
    yield 42


async def bad():
    yield from [1, 2, 3] # [yield-inside-async-function]



"""
'typing.NoReturn' is broken inside compound types for Python 3.7.0
https://bugs.python.org/issue34921

If no runtime introspection is required, use string annotations instead.
"""
# pylint: disable=missing-docstring,broad-exception-raised,invalid-name
import typing
from typing import TYPE_CHECKING, Callable, NoReturn, Union

import typing_extensions


def func1() -> NoReturn:
    raise Exception


def func2() -> Union[None, NoReturn]:  # [broken-noreturn]
    pass


def func3() -> Union[None, "NoReturn"]:
    pass


def func4() -> Union[None, typing.NoReturn]:  # [broken-noreturn]
    pass


def func5() -> Union[None, typing_extensions.NoReturn]:  # [broken-noreturn]
    pass


Alias1 = NoReturn
Alias2 = Callable[..., NoReturn]  # [broken-noreturn]
Alias3 = Callable[..., "NoReturn"]

if TYPE_CHECKING:
    # ok inside TYPE_CHECKING block
    Alias4 = Callable[..., NoReturn]



"""Make sure warnings about redefinitions do not trigger for dummy variables."""


_, INTERESTING = 'a=b'.split('=')

value = 10  # [invalid-name]


def clobbering():
    """Clobbers a dummy name from the outer scope."""
    value = 9  # [redefined-outer-name]
    for _ in range(7):
        print(value)



"""Tests for dataclass and checks that check for parameters."""

import dataclasses
from dataclasses import KW_ONLY as keyword_only
from dataclasses import dataclass


@dataclass
class MyDataClass:
    """Simple dataclass with a KW_ONLY parameter."""

    _: dataclasses.KW_ONLY
    data: str


MyDataClass(data="test")


@dataclass
class MyDataClassWithAliases:
    """Simple dataclass with an aliased KW_ONLY parameter."""

    _: keyword_only
    data: str


MyDataClassWithAliases(data="test")



"""AttributeError: 'Subscript' object has no attribute 'name' """
# pylint: disable=missing-docstring

# Disabled because of a bug with pypy 3.8 see
# https://github.com/pylint-dev/pylint/pull/7918#issuecomment-1352737369
# pylint: disable=multiple-statements

from typing import Optional

from attr import attrib, attrs  # pylint: disable=import-error


@attrs()
class User:
    name: str = attrib()
    age: int = attrib()
    occupation = Optional[str] = attrib(default=None)  # [unsupported-assignment-operation]



#pylint: disable= missing-module-docstring

def foobar1(one: int, /, two: str, *, three: float) -> int:
    """Description of the function

    Args:
        one: A number.
        two: Another number.
        three: Yes another number.

    Returns:
        The number one.
    """
    print(one, two, three)
    return 1


def foobar2(one, /, two, * three):
    # type: (int, str, float) -> int
    """Description of the function

    Args:
        one: A number.
        two: Another number.
        three: Yes another number.

    Returns:
        The number one.
    """
    print(one, two, three)
    return 1


def foobar3(
    one,  # type: int
    /,
    two,  # type: str
    *,
    three,  # type: float
):
    # type: (...) -> int
    """Description of the function

    Args:
        one: A number.
        two: Another number.
        three: Yes another number.

    Returns:
        The number one.
    """
    print(one, two, three)
    return 1



"""
This file is used for bad-option-value's test. We needed a module that isn’t restricted by line numbers
as the tests use various line numbers to test the behaviour.

Using an empty module creates issue as you can’t disable something on line 10 if it doesn’t exist. Thus, we created an extra long file so we never run into an issue with that.
"""





























































































print(1)



"""Tests for missing-param-doc and missing-type-doc for Google style docstrings
with accept-no-param-doc = no

Styleguide:
https://google.github.io/styleguide/pyguide.html#doc-function-args
"""

# pylint: disable=invalid-name, unused-argument, undefined-variable
# pylint: disable=line-too-long, too-few-public-methods, missing-class-docstring
# pylint: disable=missing-function-docstring, function-redefined, inconsistent-return-statements
# pylint: disable=dangerous-default-value, too-many-arguments

from __future__ import annotations


def test_multi_line_parameters(param: int) -> None:
    """Checks that multi line parameters lists are checked correctly
    See https://github.com/pylint-dev/pylint/issues/5452

    Args:
        param:
            a description
    """
    print(param)


def test_missing_func_params_in_google_docstring(  # [missing-param-doc, missing-type-doc]
    x, y, z
):
    """Example of a function with missing Google style parameter
    documentation in the docstring

    Args:
        x: bla
        z (int): bar

    some other stuff
    """


def test_missing_func_params_with_annotations_in_google_docstring(x: int, y: bool, z):
    """Example of a function with missing Google style parameter
    documentation in the docstring.

        Args:
            x: bla
            y: blah blah
            z (int): bar

        some other stuff
    """


def test_missing_type_doc_google_docstring_exempt_kwonly_args(
    arg1: int, arg2: int, *, value1: str, value2: str
):
    """Code to show failure in missing-type-doc

    Args:
        arg1: First argument.
        arg2: Second argument.
        value1: First kwarg.
        value2: Second kwarg.
    """
    print("NOTE: It doesn't like anything after the '*'.")


def test_default_arg_with_annotations_in_google_docstring(
    x: int, y: bool, z: int = 786
):
    """Example of a function with missing Google style parameter
        documentation in the docstring.

    Args:
        x: bla
        y: blah blah
        z: bar

    some other stuff
    """


def test_missing_func_params_with_partial_annotations_in_google_docstring(  # [missing-type-doc]
    x, y: bool, z
):
    """Example of a function with missing Google style parameter
    documentation in the docstring.

    Args:
        x: bla
        y: blah blah
        z (int): bar

    some other stuff
    """


def test_non_builtin_annotations_in_google_docstring(
    bottomleft: Point, topright: Point
) -> float:
    """Example of a function with missing Google style parameter
    documentation in the docstring.
        Args:
            bottomleft: bottom left point of rectangle
            topright: top right point of rectangle
    """


def test_non_builtin_annotations_for_returntype_in_google_docstring(
    bottomleft: Point, topright: Point
) -> Point:
    """Example of a function with missing Google style parameter
    documentation in the docstring.
    Args:
        bottomleft: bottom left point of rectangle
        topright: top right point of rectangle
    """


def test_func_params_and_keyword_params_in_google_docstring(this, other, that=True):
    """Example of a function with Google style parameter split
    in Args and Keyword Args in the docstring

        Args:
            this (str): Printed first
            other (int): Other args

        Keyword Args:
            that (bool): Printed second
    """
    print(this, that, other)


def test_func_params_and_wrong_keyword_params_in_google_docstring(  # [missing-param-doc, missing-type-doc, differing-param-doc, differing-type-doc]
    this, other, that=True
):
    """Example of a function with Google style parameter split
    in Args and Keyword Args in the docstring but with wrong keyword args

        Args:
            this (str): Printed first
            other (int): Other args

        Keyword Args:
            these (bool): Printed second
    """
    print(this, that, other)


class Foo:
    def test_missing_method_params_in_google_docstring(  # [missing-param-doc, missing-type-doc]
        self, x, y
    ):
        """Example of a class method with missing parameter documentation in
        the Google style docstring

        missing parameter documentation

        Args:
            x: bla
        """


def test_existing_func_params_in_google_docstring(xarg, yarg, zarg, warg):
    """Example of a function with correctly documented parameters and
    return values (Google style)

    Args:
        xarg (int): bla xarg
        yarg (my.qualified.type): bla
            bla yarg

        zarg (int): bla zarg
        warg (my.qualified.type): bla warg

    Returns:
        float: sum
    """
    return xarg + yarg


def test_wrong_name_of_func_params_in_google_docstring_one(  # [missing-param-doc, missing-type-doc, differing-param-doc, differing-type-doc]
    xarg, yarg, zarg
):
    """Example of functions with inconsistent parameter names in the
    signature and in the Google style documentation

    Args:
        xarg1 (int): bla xarg
        yarg (float): bla yarg

        zarg1 (str): bla zarg
    """
    return xarg + yarg


def test_wrong_name_of_func_params_in_google_docstring_two(  # [differing-param-doc, differing-type-doc]
    xarg, yarg
):
    """Example of functions with inconsistent parameter names in the
    signature and in the Google style documentation

    Args:
        yarg1 (float): bla yarg

    For the other parameters, see bla.
    """
    return xarg + yarg


def test_see_sentence_for_func_params_in_google_docstring(xarg, yarg):
    """Example for the usage of "For the other parameters, see" to avoid
        too many repetitions, e.g. in functions or methods adhering to a
        given interface (Google style)

    Args:
        yarg (float): bla yarg

    For the other parameters, see :func:`bla`
    """
    return xarg + yarg


class ClassFoo:  # [missing-param-doc, missing-type-doc]
    """test_constr_params_in_class_google
    Example of a class with missing constructor parameter documentation
    (Google style)

    Everything is completely analogous to functions.

    Args:
        y: bla

    missing constructor parameter documentation
    """

    def __init__(self, x, y):
        pass


class ClassFoo:
    def __init__(self, x, y):  # [missing-param-doc, missing-type-doc]
        """test_constr_params_in_init_google
        Example of a class with missing constructor parameter documentation
        (Google style)

        Args:
            y: bla

        missing constructor parameter documentation
        """


class ClassFoo:  # [multiple-constructor-doc,missing-param-doc, missing-type-doc]
    """test_constr_params_in_class_and_init_google
    Example of a class with missing constructor parameter documentation
    in both the init docstring and the class docstring
    (Google style)

    Everything is completely analogous to functions.

    Args:
        y: bla

    missing constructor parameter documentation
    """

    def __init__(self, x, y):  # [missing-param-doc, missing-type-doc]
        """docstring foo

        Args:
            y: bla

        missing constructor parameter documentation
        """


def test_warns_missing_args_google(named_arg, *args):  # [missing-param-doc]
    """The docstring

    Args:
        named_arg (object): Returned

    Returns:
        object or None: Maybe named_arg
    """
    if args:
        return named_arg


def test_warns_missing_kwargs_google(named_arg, **kwargs):  # [missing-param-doc]
    """The docstring

    Args:
        named_arg (object): Returned

    Returns:
        object or None: Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_finds_args_without_type_google(named_arg, *args):
    """The docstring

    Args:
        named_arg (object): Returned
        *args: Optional arguments

    Returns:
        object or None: Maybe named_arg
    """
    if args:
        return named_arg


def test_finds_kwargs_without_type_google(named_arg, **kwargs):
    """The docstring

    Args:
        named_arg (object): Returned
        **kwargs: Keyword arguments

    Returns:
        object or None: Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_finds_kwargs_without_type_google(named_arg, **kwargs: dict[str, str]):
    """The docstring

    Args:
        named_arg (object): Returned
        **kwargs: Keyword arguments

    Returns:
        object or None: Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_finds_kwargs_without_asterisk_google(named_arg, **kwargs):
    """The docstring

    Args:
        named_arg (object): Returned
        kwargs: Keyword arguments

    Returns:
        object or None: Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_finds_escaped_args_google(value: int, *args: Any) -> None:
    """This is myfunc.

    Args:
        \\*args: this is args
        value: this is value
    """
    print(*args, value)


def test_finds_args_with_xref_type_google(named_arg, **kwargs):
    """The docstring

    Args:
        named_arg (`example.value`): Returned
        **kwargs: Keyword arguments

    Returns:
        `example.value`: Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_ignores_optional_specifier_google(
    param1, param2, param3=(), param4=[], param5=[], param6=True
):
    """Do something.

    Args:
        param1 (str): Description.
        param2 (dict(str, int)): Description.
        param3 (tuple(str), optional): Defaults to empty. Description.
        param4 (List[str], optional): Defaults to empty. Description.
        param5 (list[tuple(str)], optional): Defaults to empty. Description.
        param6 (bool, optional): Defaults to True. Description.

    Returns:
        int: Description.
    """
    return param1, param2, param3, param4, param5, param6


def test_finds_multiple_complex_types_google(
    named_arg_one,
    named_arg_two,
    named_arg_three,
    named_arg_four,
    named_arg_five,
    named_arg_six,
    named_arg_seven,
    named_arg_eight,
    named_arg_nine,
    named_arg_ten,
):
    """The google docstring

    Args:
        named_arg_one (dict(str, str)): Returned
        named_arg_two (dict[str, str]): Returned
        named_arg_three (int or str): Returned
        named_arg_four (tuple(int or str)): Returned
        named_arg_five (tuple(int) or list(int)): Returned
        named_arg_six (tuple(int or str) or list(int or str)): Returned
        named_arg_seven (dict(str,str)): Returned
        named_arg_eight (dict[str,str]): Returned
        named_arg_nine (tuple(int)): Returned
        named_arg_ten (list[tokenize.TokenInfo]): Returned

    Returns:
        dict(str, str): named_arg_one
        dict[str, str]: named_arg_two
        int or str: named_arg_three
        tuple(int or str): named_arg_four
        tuple(int) or list(int): named_arg_five
        tuple(int or str) or list(int or str): named_arg_six
        dict(str,str): named_arg_seven
        dict[str,str]: named_arg_eight
        tuple(int): named_arg_nine
        list[tokenize.TokenInfo]: named_arg_ten
    """
    return (
        named_arg_one,
        named_arg_two,
        named_arg_three,
        named_arg_four,
        named_arg_five,
        named_arg_six,
        named_arg_seven,
        named_arg_eight,
        named_arg_nine,
        named_arg_ten,
    )

def test_escape_underscore(something: int, raise_: bool = False) -> bool:
    """Tests param with escaped _ is handled correctly.

    Args:
        something: the something
        raise\\_: the other

    Returns:
        something
    """
    return something and raise_



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

import os
import re
import sys
from pathlib import Path

import astroid

from pylint.checkers import variables
from pylint.interfaces import HIGH
from pylint.testutils import CheckerTestCase, MessageTest, linter, set_config
from pylint.testutils.reporter_for_tests import GenericTestReporter

REGR_DATA_DIR = str(Path(__file__).parent / ".." / "regrtest_data")


class TestVariablesChecker(CheckerTestCase):
    CHECKER_CLASS = variables.VariablesChecker

    def test_all_elements_without_parent(self) -> None:
        node = astroid.extract_node("__all__ = []")
        node.value.elts.append(astroid.Const("test"))
        root = node.root()
        with self.assertNoMessages():
            self.checker.visit_module(root)
            self.checker.leave_module(root)


class TestVariablesCheckerWithTearDown(CheckerTestCase):
    CHECKER_CLASS = variables.VariablesChecker

    def setup_method(self) -> None:
        super().setup_method()
        self._to_consume_backup = self.checker._to_consume
        self.checker._to_consume = []

    def teardown_method(self) -> None:
        self.checker._to_consume = self._to_consume_backup

    @set_config(callbacks=("callback_", "_callback"))
    def test_custom_callback_string(self) -> None:
        """Test the --callbacks option works."""
        node = astroid.extract_node(
            """
        def callback_one(abc):
             ''' should not emit unused-argument. '''
        """
        )
        with self.assertNoMessages():
            self.checker.visit_functiondef(node)
            self.checker.leave_functiondef(node)

        node = astroid.extract_node(
            """
        def two_callback(abc, defg):
             ''' should not emit unused-argument. '''
        """
        )
        with self.assertNoMessages():
            self.checker.visit_functiondef(node)
            self.checker.leave_functiondef(node)

        node = astroid.extract_node(
            """
        def normal_func(abc):
             ''' should emit unused-argument. '''
        """
        )
        with self.assertAddsMessages(
            MessageTest(
                "unused-argument",
                node=node["abc"],
                args="abc",
                confidence=HIGH,
                line=2,
                col_offset=16,
                end_line=2,
                end_col_offset=19,
            )
        ):
            self.checker.visit_functiondef(node)
            self.checker.leave_functiondef(node)

        node = astroid.extract_node(
            """
        def cb_func(abc):
             ''' Previous callbacks are overridden. '''
        """
        )
        with self.assertAddsMessages(
            MessageTest(
                "unused-argument",
                node=node["abc"],
                args="abc",
                confidence=HIGH,
                line=2,
                col_offset=12,
                end_line=2,
                end_col_offset=15,
            )
        ):
            self.checker.visit_functiondef(node)
            self.checker.leave_functiondef(node)

    @set_config(redefining_builtins_modules=("os",))
    def test_redefined_builtin_modname_not_ignored(self) -> None:
        node = astroid.parse(
            """
        from future.builtins import open
        """
        )
        with self.assertAddsMessages(
            MessageTest(
                "redefined-builtin",
                node=node.body[0],
                args="open",
                line=2,
                col_offset=0,
                end_line=2,
                end_col_offset=32,
            )
        ):
            self.checker.visit_module(node)

    @set_config(redefining_builtins_modules=("os",))
    def test_redefined_builtin_in_function(self) -> None:
        node = astroid.extract_node(
            """
        def test():
            from os import open
        """
        )
        with self.assertNoMessages():
            self.checker.visit_module(node.root())
            self.checker.visit_functiondef(node)

    def test_import_as_underscore(self) -> None:
        node = astroid.parse(
            """
        import math as _
        """
        )
        with self.assertNoMessages():
            self.walk(node)

    def test_lambda_in_classdef(self) -> None:
        # Make sure lambda doesn't raises
        # Undefined-method in class def

        # Issue 1824
        # https://github.com/pylint-dev/pylint/issues/1824
        node = astroid.parse(
            """
        class MyObject(object):
            method1 = lambda func: func()
            method2 = lambda function: function()
        """
        )
        with self.assertNoMessages():
            self.walk(node)

    def test_nested_lambda(self) -> None:
        """Make sure variables from parent lambdas
        aren't noted as undefined

        https://github.com/pylint-dev/pylint/issues/760
        """
        node = astroid.parse(
            """
        lambda x: lambda: x + 1
        """
        )
        with self.assertNoMessages():
            self.walk(node)

    @set_config(ignored_argument_names=re.compile("arg"))
    def test_ignored_argument_names_no_message(self) -> None:
        """Make sure is_ignored_argument_names properly ignores
        function arguments
        """
        node = astroid.parse(
            """
        def fooby(arg):
            pass
        """
        )
        with self.assertNoMessages():
            self.walk(node)

    @set_config(ignored_argument_names=re.compile("args|kwargs"))
    def test_ignored_argument_names_starred_args(self) -> None:
        node = astroid.parse(
            """
        def fooby(*args, **kwargs):
            pass
        """
        )
        with self.assertNoMessages():
            self.walk(node)


class TestMissingSubmodule(CheckerTestCase):
    CHECKER_CLASS = variables.VariablesChecker

    @staticmethod
    def test_package_all() -> None:
        sys.path.insert(0, REGR_DATA_DIR)
        try:
            linter.check([os.path.join(REGR_DATA_DIR, "package_all")])
            assert isinstance(linter.reporter, GenericTestReporter)
            got = linter.reporter.finalize().strip()
            assert got == "E:  3: Undefined variable name 'missing' in __all__"
        finally:
            sys.path.pop(0)



"""Tests for missing-yield-doc and missing-yield-type-doc for Google style docstrings"""
# pylint: disable=missing-function-docstring, unused-argument, function-redefined
# pylint: disable=invalid-name, undefined-variable
import typing


# Test redundant yields docstring variants
def my_func(self):
    """This is a docstring.

    Yields:
        int or None: One, or sometimes None.
    """
    if a_func():
        yield None
    yield 1


def my_func(self):  # [redundant-yields-doc]
    """This is a docstring.

    Yields:
        int: One
    """
    return 1


# Test missing yields typing docstring
def generator() -> typing.Iterator[int]:
    """A simple function for checking type hints.

    Yields:
        The number 0
    """
    yield 0



"""Tests for the mixin-class-rgx option"""
# pylint: disable=too-few-public-methods


# Tests for not-async-context-manager


class AsyncManagerMixedin:
    """Class that does not match the option pattern"""

    def __aenter__(self):
        pass


class AsyncManagerMixin:
    """Class that does match the option pattern"""

    def __aenter__(self):
        pass


async def check_not_async_context_manager():
    """Function calling the classes for not-async-context-manager"""
    async with AsyncManagerMixedin:  # [not-async-context-manager]
        pass
    async with AsyncManagerMixin():
        pass


# Tests for attribute-defined-outside-init


class OutsideInitMixedin:
    """Class that does not match the option pattern"""

    def set_attribute(self):
        """Set an attribute outside of __init__"""
        self.attr = 1  # [attribute-defined-outside-init]


class OutsideInitMixin:
    """Class that does match the option pattern"""

    def set_attribute(self):
        """Set an attribute outside of __init__"""
        self.attr = 1


# Tests for no-member


class NoMemberMixedin:
    """Class that does not match the option pattern"""

MY_CLASS = OutsideInitMixedin().method() # [no-member]

class NoMemberMixin:
    """Class that does match the option pattern"""

MY_OTHER_CLASS = NoMemberMixin().method()



""""Tests for return type checkers for protocol methods with ellipsis function body"""
# pylint: disable=missing-class-docstring
from typing import Any, Iterator


class MyClass:
    """The "invalid-*-returned" messages shouldn't be emitted for stub functions
    Original issue: https://github.com/pylint-dev/pylint/issues/4736"""

    def __len__(self) -> int:
        ...

    def __hash__(self) -> int:
        ...

    def __index__(self) -> int:
        ...

    def __iter__(self) -> Iterator[Any]:
        ...

    def __bool__(self) -> bool:
        ...

    def __repr__(self) -> object:
        ...

    def __str__(self) -> str:
        ...

    def __bytes__(self) -> bytes:
        ...

    def __length_hint__(self) -> int:
        ...

    def __format__(self, format_spec: str) -> str:
        ...

    def __getnewargs__(self) -> tuple:
        ...

    def __getnewargs_ex__(self) -> tuple:
        ...



"""Tests for missing-param-doc and missing-type-doc for Numpy style docstrings
with accept-no-param-doc = no
"""

# pylint: disable=invalid-name, unused-argument, undefined-variable, too-many-arguments
# pylint: disable=line-too-long, too-few-public-methods, missing-class-docstring
# pylint: disable=missing-function-docstring, function-redefined, inconsistent-return-statements

from __future__ import annotations


def test_missing_func_params_in_numpy_docstring(  # [missing-param-doc, missing-type-doc]
    x, y, z
):
    """Example of a function with missing NumPy style parameter
        documentation in the docstring

    Parameters
    ----------
    x:
        bla
    z: int
        bar

    some other stuff
    """


class Foo:
    def test_missing_method_params_in_numpy_docstring(  # [missing-param-doc, missing-type-doc]
        self, x, y
    ):
        """Example of a class method with missing parameter documentation in
        the Numpy style docstring

        missing parameter documentation

        Parameters
        ----------
        x:
            bla
        """


def test_existing_func_params_in_numpy_docstring(xarg, yarg, zarg, warg):
    """Example of a function with correctly documented parameters and
    return values (Numpy style)

    Parameters
    ----------
    xarg: int
        bla xarg
    yarg: my.qualified.type
        bla yarg

    zarg: int
        bla zarg
    warg: my.qualified.type
        bla warg

    Returns
    -------
    float
        sum
    """
    return xarg + yarg


def test_wrong_name_of_func_params_in_numpy_docstring(  # [missing-param-doc, missing-type-doc, differing-param-doc, differing-type-doc]
    xarg, yarg, zarg
):
    """Example of functions with inconsistent parameter names in the
    signature and in the Numpy style documentation

    Parameters
    ----------
    xarg1: int
        bla xarg
    yarg: float
        bla yarg

    zarg1: str
        bla zarg
    """
    return xarg + yarg


def test_wrong_name_of_func_params_in_numpy_docstring_two(  # [differing-param-doc, differing-type-doc]
    xarg, yarg
):
    """Example of functions with inconsistent parameter names in the
    signature and in the Numpy style documentation

    Parameters
    ----------
    yarg1: float
        bla yarg

    For the other parameters, see bla.
    """
    return xarg + yarg


def test_see_sentence_for_func_params_in_numpy_docstring(xarg, yarg):
    """Example for the usage of "For the other parameters, see" to avoid
    too many repetitions, e.g. in functions or methods adhering to a
    given interface (Numpy style)

    Parameters
    ----------
    yarg: float
        bla yarg

    For the other parameters, see :func:`bla`
    """
    return xarg + yarg


class ClassFoo:  # [missing-param-doc, missing-type-doc]
    """test_constr_params_in_class_numpy
    Example of a class with missing constructor parameter documentation
    (Numpy style)

    Everything is completely analogous to functions.

    Parameters
    ----------
    y:
        bla

    missing constructor parameter documentation
    """

    def __init__(self, x, y):
        pass


class ClassFoo:
    """test_constr_params_and_attributes_in_class_numpy
    Example of a class with correct constructor parameter documentation
    and an attributes section (Numpy style)

    Parameters
    ----------
    foobar : str
        Something.

    Attributes
    ----------
    barfoor : str
        Something.
    """

    def __init__(self, foobar):
        self.barfoo = None


class ClassFoo:
    def __init__(self, x, y):  # [missing-param-doc, missing-type-doc]
        """test_constr_params_in_init_numpy
        Example of a class with missing constructor parameter documentation
        (Numpy style)

        Everything is completely analogous to functions.

        Parameters
        ----------
        y:
            bla

        missing constructor parameter documentation
        """


class ClassFoo:  # [multiple-constructor-doc, missing-param-doc, missing-type-doc]
    """test_constr_params_in_class_and_init_numpy
    Example of a class with missing constructor parameter documentation
    in both the init docstring and the class docstring
    (Numpy style)

    Everything is completely analogous to functions.

    Parameters
    ----------
    y:
        bla

    missing constructor parameter documentation
    """

    def __init__(self, x, y):  # [missing-param-doc, missing-type-doc]
        """docstring foo

        Parameters
        ----------
        y:
            bla

        missing constructor parameter documentation
        """


def test_warns_missing_args_numpy(named_arg, *args):  # [missing-param-doc]
    """The docstring

    Args
    ----
    named_arg : object
        Returned

    Returns
    -------
        object or None
            Maybe named_arg
    """
    if args:
        return named_arg


def test_warns_missing_kwargs_numpy(named_arg, **kwargs):  # [missing-param-doc]
    """The docstring

    Args
    ----
    named_arg : object
        Returned

    Returns
    -------
        object or None
            Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_finds_args_without_type_numpy(  # [missing-type-doc]
    named_arg, typed_arg: bool, untyped_arg, *args
):
    """The docstring

    Args
    ----
    named_arg : object
        Returned
    typed_arg
        Other argument without numpy type annotation
    untyped_arg
        Other argument without any type annotation
    *args :
        Optional Arguments

    Returns
    -------
        object or None
            Maybe named_arg
    """
    if args:
        return named_arg


def test_finds_args_with_xref_type_numpy(named_arg, *args):
    """The docstring

    Args
    ----
    named_arg : `example.value`
        Returned
    *args :
        Optional Arguments

    Returns
    -------
        `example.value`
            Maybe named_arg
    """
    if args:
        return named_arg


def test_finds_kwargs_without_type_numpy(named_arg, **kwargs):
    """The docstring

    Args
    ----
    named_arg : object
        Returned
    **kwargs :
        Keyword arguments

    Returns
    -------
        object or None
            Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_finds_kwargs_with_type_numpy(named_arg, **kwargs: dict[str, str]):
    """The docstring

    Args
    ----
    named_arg : object
        Returned
    **kwargs :
        Keyword arguments

    Returns
    -------
        object or None
            Maybe named_arg
    """
    if kwargs:
        return named_arg


def test_finds_kwargs_without_asterisk_numpy(named_arg, **kwargs):
    """The docstring

    Args
    ----
    named_arg : object
        Returned
    kwargs :
        Keyword arguments

    Returns
    -------
        object or None
            Maybe named_arg
    """
    if kwargs:
        return named_arg


def my_func(
    named_arg_one,
    named_arg_two,
    named_arg_three,
    named_arg_four,
    named_arg_five,
    named_arg_six,
    named_arg_seven,
    named_arg_eight,
):
    """The docstring

    Args
    ----
    named_arg_one : dict(str,str)
        Returned
    named_arg_two : dict[str,str]
        Returned
    named_arg_three : tuple(int)
        Returned
    named_arg_four : list[tokenize.TokenInfo]
        Returned
    named_arg_five : int or str
        Returned
    named_arg_six : tuple(int or str)
        Returned
    named_arg_seven : tuple(int) or list(int)
        Returned
    named_arg_eight : tuple(int or str) or list(int or str)
        Returned

    Returns
    -------
        dict(str,str)
            named_arg_one
        dict[str,str]
            named_arg_two
        tuple(int)
            named_arg_three
        list[tokenize.TokenInfo]
            named_arg_four
        int or str
            named_arg_five
        tuple(int or str)
            named_arg_six
        tuple(int) or list(int)
            named_arg_seven
        tuple(int or str) or list(int or str)
            named_arg_eight
    """
    return (
        named_arg_one,
        named_arg_two,
        named_arg_three,
        named_arg_four,
        named_arg_five,
        named_arg_six,
        named_arg_seven,
        named_arg_eight,
    )


def test_ignores_optional_specifier_numpy(param, param2="all"):
    """Do something.

    Parameters
    ----------
    param : str
        Description.
    param2 : str, optional
        Description (the default is 'all').

    Returns
    -------
    int
        Description.
    """
    return param, param2


def test_with_list_of_default_values(arg, option, option2):
    """Reported in https://github.com/pylint-dev/pylint/issues/4035.

    Parameters
    ----------
    arg : int
        The number of times to print it.
    option : {"y", "n"}
        Do I do it?
    option2 : {"y", None, "n"}
        Do I do it?

    """
    return arg, option, option2


def test_with_descriptions_instead_of_typing(arg, axis, option):
    """We choose to accept description in place of typing as well.

    See: https://github.com/pylint-dev/pylint/pull/7398.

    Parameters
    ----------
    arg : a number type.
    axis : int or None
    option : {"y", "n"}
        Do I do it?
    """
    return arg, option



"""Test the superfluous-parens warning with python 3.8 functionality (walrus operator)"""
# pylint: disable=missing-function-docstring, invalid-name, missing-class-docstring, import-error, pointless-statement,named-expr-without-context
import numpy

# Test parens in if statements
if not (x := False):
    print(x)

if not (foo := 5):
    pass

A = 1
if odd := isinstance(A, int):
    pass

if not ((x := 1)):  # [superfluous-parens]
    pass

if ((x := A)):   # [superfluous-parens]
    pass

if not ((x := A)):  # [superfluous-parens]
    pass

if not ((((x := 1)))):  # [superfluous-parens]
    pass

# Test assertions
assert (ret := str(42))

# Teast 2D arrays with numpy (since they allow commas within indexing)
ARRAY_2D = numpy.zeros((3, 3), dtype=bool)
E = not (vals := ARRAY_2D[:, :].all())
F = not (vals := ARRAY_2D[2, 2].all())
G = not (vals := ARRAY_2D[::].all())
H = not (vals := ARRAY_2D[2:2].all())

# Test yield
class TestYieldClass:
    @classmethod
    def function_A(cls):
        yield (var := 1 + 1)
        print(var)

    @classmethod
    def function_B(cls):
        yield str(1 + 1)

    @classmethod
    def function_C(cls):
        yield (1 + 1) # [superfluous-parens]


if (x := "Test " + "String"):
    print(x)

if (x := ("Test " + "String")):  # [superfluous-parens]
    print(x)

if not (foo := "Test " + "String" in "hello"):
    print(foo)

if not (foo := ("Test " + "String") in "hello"):  # [superfluous-parens]
    print(foo)

assert (ret := "Test " + "String")
assert (ret := ("Test " + "String"))  # [superfluous-parens]

(walrus := False)
(walrus := (False))  # [superfluous-parens]

(hi := ("CONST"))  # [superfluous-parens]
(hi := ("CONST",))



#pylint: disable = missing-any-param-doc
"""demonstrate FP with useless-type-doc"""


def function(public_param: int, _some_private_param: bool = False) -> None:
    """does things

    Args:
        public_param: an ordinary parameter
    """
    for _ in range(public_param):
        ...
    if _some_private_param:
        ...
    else:
        ...


def smart_function(public_param: int, _some_private_param: bool = False) -> None:
    """We're speaking about _some_private_param without really documenting it.

    Args:
        public_param: an ordinary parameter
    """
    for _ in range(public_param):
        ...
    if _some_private_param:
        ...
    else:
        ...


# +1: [useless-type-doc,useless-param-doc]
def function_useless_doc(public_param: int, _some_private_param: bool = False) -> None:
    """does things

    Args:
        public_param: an ordinary parameter
        _some_private_param (bool): private param

    """
    for _ in range(public_param):
        ...
    if _some_private_param:
        ...
    else:
        ...


def test(_new: str) -> str:
    """foobar

    :return: comment
    """
    return ""


def smarter_test(_new: str) -> str:
    """We're speaking about _new without really documenting it.

    :return: comment
    """
    return ""


# +1: [useless-type-doc,useless-param-doc]
def test_two(_new: str) -> str:
    """foobar

    :param str _new:
    :return: comment
    """
    return ""



# pylint: disable=missing-docstring, pointless-statement, useless-object-inheritance
# pylint: disable=duplicate-bases, too-few-public-methods


class Klass(object, object):
    def get(self):
        self._non_existent_attribute # [no-member]



"""Tests for import-private-name."""
# pylint: disable=unused-import, missing-docstring, reimported, import-error, wrong-import-order
# pylint: disable=no-name-in-module, multiple-imports, ungrouped-imports, misplaced-future
# pylint: disable=wrong-import-position, relative-beyond-top-level

# Basic cases
from _world import hello  # [import-private-name]
from _world import _hello  # [import-private-name]
from city import _house  # [import-private-name]
from city import a, _b, c, _d  # [import-private-name]
from _city import a, _b, c, _d  # [import-private-name]
from city import a, b, c, _d  # [import-private-name]
import house
import _house  # [import-private-name]
import _house, _chair, _stair  # [import-private-name]
import house, _chair, _stair  # [import-private-name]

# Ignore dunders
import __asd__
import __future__
from __future__ import print_function
from __future__ import __print_function__

# Ignore local modules
# The check for local modules compares directory names in the path of the file being linted to
# the name of the module we are importing from. The use of `__init__.py` to indicate Python modules
# is deprecated so this is a heuristic solution.
# If we were importing from `pylint`, it would be counted as a valid internal private import
# and not emit a message as long as this file has a parent directory called `pylint`, even though
# we are not importing from that directory. (We would be importing from `pylint/pylint`.)
from private_import import _private  # pylint: disable=import-self
from private_import.other_file import _private
from . import _private
from astroid import _private  # [import-private-name]
from sys import _private  # [import-private-name]

# Ignore typecheck
from typing import TYPE_CHECKING, List, Optional

if TYPE_CHECKING:
    import _TreeType
    from types import _TreeType
    from _types import TreeType
    from _types import _TreeType

# No error since imports are used as type annotations
from classes import _PrivateClassA, safe_get_A
from classes import _PrivateClassB
from classes import _PrivateClassC

a_var: _PrivateClassA = safe_get_A()

def b_func(class_b: _PrivateClassB):
    print(class_b)

def c_func() -> _PrivateClassC:
    return None

# Used as typing in slices
from classes import _SubScriptA
from classes import _SubScriptB

a: Optional[_SubScriptA]
b: Optional[_SubScriptB[List]]

import _TypeContainerA
import _TypeContainerB
import _TypeContainerC

import SafeContainerA
a2: _TypeContainerA.A = SafeContainerA.safe_get_a()

def b_func2(class_b2: _TypeContainerB.B):
    print(class_b2)

def c2_func() -> _TypeContainerC.C:
    return None

# This is allowed since all the imports are used for typing
from _TypeContainerExtra import TypeExtraA, TypeExtraB
from MakerContainerExtra import GetA, GetB
extraA: TypeExtraA = GetA()
extraB: TypeExtraB = GetB()

# This is not allowed because there is an import not used for typing
from _TypeContainerExtra2 import TypeExtra2, NotTypeExtra # [import-private-name]
extra2: TypeExtra2

# Try many cases to ensure that type annotation usages of a private import
# do not mask other illegal usages of the import
import _private_module # [import-private-name]
my_var: _private_module.Thing = _private_module.Thing()

import _private_module2 # [import-private-name]
my_var2: _private_module2.Thing2
my_var2 = _private_module2.Thing2()

import _private_module3 # [import-private-name]
my_var3: _private_module3.Thing3
my_var3 = _private_module3.Thing3
my_var3_2: _private_module3.Thing3

import _private_module4 # [import-private-name]
my_var4: _private_module4.Thing4
my_var4 = _private_module4.get_callers().get_thing4()

from _private_module5 import PrivateClass # [import-private-name]
my_var5: PrivateClass
my_var5 = PrivateClass()

from _private_module6 import PrivateClass2 # [import-private-name]
my_var6: PrivateClass2 = PrivateClass2()

from public_module import _PrivateClass3 # [import-private-name]
my_var7: _PrivateClass3 = _PrivateClass3()

# Even though we do not see the private call, the type check does not keep us from emitting
# because we do not use that variable
import _private_module_unreachable # [import-private-name]
my_var8: _private_module_unreachable.Thing8
_private_module_unreachable.Thing8()


# pylint: disable=too-few-public-methods
class Regression6624:
    """Ensure that an import statement precedes this case."""
    def get_example(self):
        example: Example = Example().save()
        return example


class Example:
    def save(self):
        return self


# Treat relative imports as internal
from .other_file import _private
from ..parent import _private

from _private_module_x import some_name # [import-private-name]
var = some_name



# pylint: disable=missing-docstring, invalid-name, too-few-public-methods, import-outside-toplevel, fixme, line-too-long, broad-exception-raised

def test_regression_737():
    import xml # [unused-import]

def test_regression_923():
    import unittest.case  # [unused-import]
    import xml as sql # [unused-import]

def test_unused_with_prepended_underscore():
    _foo = 42
    _ = 24
    __a = 24
    dummy = 24
    _a_ = 42 # [unused-variable]
    __a__ = 24 # [unused-variable]
    __never_used = 42

def test_local_field_prefixed_with_unused_or_ignored():
    flagged_local_field = 42 # [unused-variable]
    unused_local_field = 42
    ignored_local_field = 42


class HasUnusedDunderClass:

    def test(self):
        __class__ = 42  # [unused-variable]

    def best(self):
        self.test()


def locals_example_defined_before():
    value = 42  # [possibly-unused-variable]
    return locals()


def locals_example_defined_after():
    local_variables = locals()
    value = 42  # [unused-variable]
    return local_variables


def locals_does_not_account_for_subscopes():
    value = 42  # [unused-variable]

    def some_other_scope():
        return locals()
    return some_other_scope


def unused_import_from():
    from functools import wraps as abc # [unused-import]
    from collections import namedtuple # [unused-import]


def unused_import_in_function(value):
    from string import digits, hexdigits # [unused-import]
    return value if value in digits else "Nope"


def hello(arg):
    my_var = 'something' # [unused-variable]
    if arg:
        return True
    raise Exception

# pylint: disable=wrong-import-position
PATH = OS = collections = deque = None


def function(matches):
    """"yo"""
    aaaa = 1  # [unused-variable]
    index = -1
    for match in matches:
        index += 1
        print(match)

from astroid import nodes
def visit_if(self, node: nodes.If) -> None:
    """increments the branches counter"""
    branches = 1
    # don't double count If nodes coming from some 'elif'
    if node.orelse and len(node.orelse) > 1:
        branches += 1
    self.inc_branch(branches)
    self.stmts += branches


def test_global():
    """ Test various assignments of global
    variables through imports.
    """
    # pylint: disable=redefined-outer-name
    global PATH, OS, collections, deque  # [global-statement]
    from os import path as PATH
    import os as OS
    import collections
    from collections import deque
    # make sure that these triggers unused-variable
    from sys import platform  # [unused-import]
    from sys import version as VERSION  # [unused-import]
    import this  # [unused-import]
    import re as RE  # [unused-import]

# test cases that include exceptions
def function2():
    unused = 1  # [unused-variable]
    try:
        1 / 0
    except ZeroDivisionError as error:
        try:
            1 / 0
        except ZeroDivisionError as error:  # [redefined-outer-name]
            raise Exception("") from error

def func():
    try:
        1 / 0
    except ZeroDivisionError as error:
        try:
            1 / 0
        except error:
            print("error")

def func2():
    try:
        1 / 0
    except ZeroDivisionError as error:
        try:
            1 / 0
        except:
            raise Exception("") from error

def func3():
    try:
        1 / 0
    except ZeroDivisionError as error:
        print(f"{error}")
        try:
            1 / 2
        except TypeError as error:  # [unused-variable, redefined-outer-name]
            print("warning")

def func4():
    try:
        1 / 0
    except ZeroDivisionError as error:  # [unused-variable]
        try:
            1 / 0
        except ZeroDivisionError as error:  # [redefined-outer-name]
            print("error")


def main(lst):
    """https://github.com/pylint-dev/astroid/pull/1111#issuecomment-890367609"""
    try:
        raise ValueError
    except ValueError as e:  # [unused-variable]
        pass

    for e in lst:
        pass

    # e will be undefined if lst is empty
    print(e)  # [undefined-loop-variable]

main([])


def func5():
    """No unused-variable for a container if iterated in comprehension"""
    x = []
    # Test case requires homonym between "for x" and "in x"
    assert [True for x in x]


def sibling_except_handlers():
    try:
        pass
    except ValueError as e:
        print(e)
    try:
        pass
    except ValueError as e:
        print(e)

def func6():
    a = 1

    def nonlocal_writer():
        nonlocal a

        for a in range(10):
            pass

    nonlocal_writer()

    assert a == 9, a

def test_regression_8595():
    # pylint: disable=broad-exception-caught
    import logging
    def compute():
        pass
    try:
        compute()
        error = False
    except Exception as e:
        logging.error(e)
        error = True
    if error:
        try:
            compute()
        except Exception as e:  # [unused-variable]
            pass



"""Tests for undefined-variable related to decorators"""
# pylint: disable=missing-function-docstring, missing-class-docstring, too-few-public-methods
# pylint: disable=unnecessary-comprehension

# Test that class attributes are in scope for listcomp in decorator.
# Regression reported in https://github.com/pylint-dev/pylint/issues/511

def dec(inp):
    def inner(func):
        print(inp)
        return func
    return inner


class Cls:

    DATA = "foo"

    @dec([x for x in DATA])
    def fun(self):
        pass



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

# pylint: disable=redefined-outer-name

from __future__ import annotations

import warnings
from contextlib import redirect_stdout
from io import StringIO
from json import dumps
from pathlib import Path
from typing import TYPE_CHECKING, TextIO

import pytest

from pylint import checkers
from pylint.interfaces import HIGH
from pylint.lint import PyLinter
from pylint.message.message import Message
from pylint.reporters import BaseReporter, MultiReporter
from pylint.reporters.text import ParseableTextReporter, TextReporter
from pylint.typing import FileItem, MessageLocationTuple

if TYPE_CHECKING:
    from pylint.reporters.ureports.nodes import Section


@pytest.fixture(scope="module")
def reporter() -> type[TextReporter]:
    return TextReporter


@pytest.fixture(scope="module")
def disable() -> list[str]:
    return ["I"]


def test_template_option(linter: PyLinter) -> None:
    output = StringIO()
    linter.reporter.out = output
    linter.config.msg_template = "{msg_id}:{line:03d}"
    linter.open()
    linter.set_current_module("0123")
    linter.add_message("C0301", line=1, args=(1, 2))
    linter.add_message("line-too-long", line=2, args=(3, 4))
    assert output.getvalue() == "************* Module 0123\nC0301:001\nC0301:002\n"


def test_template_option_default(linter: PyLinter) -> None:
    """Test the default msg-template setting."""
    output = StringIO()
    linter.reporter.out = output
    linter.open()
    linter.set_current_module("my_module")
    linter.add_message("C0301", line=1, args=(1, 2))
    linter.add_message("line-too-long", line=2, args=(3, 4))

    out_lines = output.getvalue().split("\n")
    assert out_lines[1] == "my_module:1:0: C0301: Line too long (1/2) (line-too-long)"
    assert out_lines[2] == "my_module:2:0: C0301: Line too long (3/4) (line-too-long)"


def test_template_option_end_line(linter: PyLinter) -> None:
    """Test the msg-template option with end_line and end_column."""
    output = StringIO()
    linter.reporter.out = output
    linter.config.msg_template = (
        "{path}:{line}:{column}:{end_line}:{end_column}: {msg_id}: {msg} ({symbol})"
    )
    linter.open()
    linter.set_current_module("my_mod")
    linter.add_message("C0301", line=1, args=(1, 2))
    linter.add_message(
        "line-too-long", line=2, end_lineno=2, end_col_offset=4, args=(3, 4)
    )

    out_lines = output.getvalue().split("\n")
    assert out_lines[1] == "my_mod:1:0::: C0301: Line too long (1/2) (line-too-long)"
    assert out_lines[2] == "my_mod:2:0:2:4: C0301: Line too long (3/4) (line-too-long)"


def test_template_option_non_existing(linter: PyLinter) -> None:
    """Test the msg-template option with non-existent options.
    This makes sure that this option remains backwards compatible as new
    parameters do not break on previous versions
    """
    output = StringIO()
    linter.reporter.out = output
    linter.config.msg_template = "{path}:{line}:{categ}:({a_second_new_option:03d})"
    linter.open()
    with pytest.warns(UserWarning) as records:
        linter.set_current_module("my_mod")
        assert len(records) == 2
        assert "Don't recognize the argument 'categ'" in records[0].message.args[0]
    assert (
        "Don't recognize the argument 'a_second_new_option'"
        in records[1].message.args[0]
    )

    linter.add_message("C0301", line=1, args=(1, 2))
    linter.add_message(
        "line-too-long", line=2, end_lineno=2, end_col_offset=4, args=(3, 4)
    )

    out_lines = output.getvalue().split("\n")
    assert out_lines[1] == "my_mod:1::()"
    assert out_lines[2] == "my_mod:2::()"


def test_template_option_with_header(linter: PyLinter) -> None:
    output = StringIO()
    linter.reporter.out = output
    linter.config.msg_template = '{{ "Category": "{category}" }}'
    linter.open()
    linter.set_current_module("my_mod")

    linter.add_message("C0301", line=1, args=(1, 2))
    linter.add_message(
        "line-too-long", line=2, end_lineno=2, end_col_offset=4, args=(3, 4)
    )

    out_lines = output.getvalue().split("\n")
    assert out_lines[1] == '{ "Category": "convention" }'
    assert out_lines[2] == '{ "Category": "convention" }'


def test_parseable_output_deprecated() -> None:
    with warnings.catch_warnings(record=True) as cm:
        warnings.simplefilter("always")
        ParseableTextReporter()

    assert len(cm) == 1
    assert isinstance(cm[0].message, DeprecationWarning)


def test_parseable_output_regression() -> None:
    output = StringIO()
    with warnings.catch_warnings(record=True):
        warnings.simplefilter("ignore", category=DeprecationWarning)
        linter = PyLinter(reporter=ParseableTextReporter())

    checkers.initialize(linter)
    linter.config.persistent = 0
    linter.reporter.out = output
    linter.set_option("output-format", "parseable")
    linter.open()
    linter.set_current_module("0123")
    linter.add_message("line-too-long", line=1, args=(1, 2))
    assert (
        output.getvalue() == "************* Module 0123\n"
        "0123:1: [C0301(line-too-long), ] "
        "Line too long (1/2)\n"
    )


class NopReporter(BaseReporter):
    name = "nop-reporter"
    extension = ""

    def __init__(self, output: TextIO | None = None) -> None:
        super().__init__(output)
        print("A NopReporter was initialized.", file=self.out)

    def writeln(self, string: str = "") -> None:
        pass

    def _display(self, layout: Section) -> None:
        pass


def test_multi_format_output(tmp_path: Path) -> None:
    text = StringIO(newline=None)
    json = tmp_path / "somefile.json"

    source_file = tmp_path / "somemodule.py"
    source_file.write_text('NOT_EMPTY = "This module is not empty"\n')
    dumps(str(source_file))

    nop_format = NopReporter.__module__ + "." + NopReporter.__name__
    formats = ",".join(["json2:" + str(json), "text", nop_format])

    with redirect_stdout(text):
        linter = PyLinter()
        linter.load_default_plugins()
        linter.set_option("persistent", False)
        linter.set_option("reports", True)
        linter.set_option("score", True)
        linter.set_option("score", True)
        linter.set_option("output-format", formats)

        assert linter.reporter.linter is linter
        with pytest.raises(NotImplementedError):
            linter.reporter.out = text

        linter.open()
        linter.check_single_file_item(
            FileItem("somemodule", str(source_file), "somemodule")
        )
        linter.add_message("line-too-long", line=1, args=(1, 2))
        linter.generate_reports()
        linter.reporter.writeln("direct output")

        # Ensure the output files are flushed and closed
        assert isinstance(linter.reporter, MultiReporter)
        linter.reporter.close_output_files()
        del linter.reporter

    with open(json, encoding="utf-8") as f:
        assert '"messageId": "C0114"' in f.read()

    assert (
        text.getvalue() == "A NopReporter was initialized.\n"
        "************* Module somemodule\n"
        f"{source_file}:1:0: C0114: Missing module docstring (missing-module-docstring)\n"
        f"{source_file}:1:0: C0301: Line too long (1/2) (line-too-long)\n"
        "\n"
        "\n"
        "Report\n"
        "======\n"
        "1 statements analysed.\n"
        "\n"
        "Statistics by type\n"
        "------------------\n"
        "\n"
        "+---------+-------+-----------+-----------+------------+---------+\n"
        "|type     |number |old number |difference |%documented |%badname |\n"
        "+=========+=======+===========+===========+============+=========+\n"
        "|module   |1      |NC         |NC         |0.00        |0.00     |\n"
        "+---------+-------+-----------+-----------+------------+---------+\n"
        "|class    |0      |NC         |NC         |0           |0        |\n"
        "+---------+-------+-----------+-----------+------------+---------+\n"
        "|method   |0      |NC         |NC         |0           |0        |\n"
        "+---------+-------+-----------+-----------+------------+---------+\n"
        "|function |0      |NC         |NC         |0           |0        |\n"
        "+---------+-------+-----------+-----------+------------+---------+\n"
        "\n"
        "\n"
        "\n"
        "3 lines have been analyzed\n"
        "\n"
        "Raw metrics\n"
        "-----------\n"
        "\n"
        "+----------+-------+------+---------+-----------+\n"
        "|type      |number |%     |previous |difference |\n"
        "+==========+=======+======+=========+===========+\n"
        "|code      |2      |66.67 |NC       |NC         |\n"
        "+----------+-------+------+---------+-----------+\n"
        "|docstring |0      |0.00  |NC       |NC         |\n"
        "+----------+-------+------+---------+-----------+\n"
        "|comment   |0      |0.00  |NC       |NC         |\n"
        "+----------+-------+------+---------+-----------+\n"
        "|empty     |1      |33.33 |NC       |NC         |\n"
        "+----------+-------+------+---------+-----------+\n"
        "\n"
        "\n"
        "\n"
        "Duplication\n"
        "-----------\n"
        "\n"
        "+-------------------------+------+---------+-----------+\n"
        "|                         |now   |previous |difference |\n"
        "+=========================+======+=========+===========+\n"
        "|nb duplicated lines      |0     |NC       |NC         |\n"
        "+-------------------------+------+---------+-----------+\n"
        "|percent duplicated lines |0.000 |NC       |NC         |\n"
        "+-------------------------+------+---------+-----------+\n"
        "\n"
        "\n"
        "\n"
        "Messages by category\n"
        "--------------------\n"
        "\n"
        "+-----------+-------+---------+-----------+\n"
        "|type       |number |previous |difference |\n"
        "+===========+=======+=========+===========+\n"
        "|convention |2      |NC       |NC         |\n"
        "+-----------+-------+---------+-----------+\n"
        "|refactor   |0      |NC       |NC         |\n"
        "+-----------+-------+---------+-----------+\n"
        "|warning    |0      |NC       |NC         |\n"
        "+-----------+-------+---------+-----------+\n"
        "|error      |0      |NC       |NC         |\n"
        "+-----------+-------+---------+-----------+\n"
        "\n"
        "\n"
        "\n"
        "Messages\n"
        "--------\n"
        "\n"
        "+-------------------------+------------+\n"
        "|message id               |occurrences |\n"
        "+=========================+============+\n"
        "|missing-module-docstring |1           |\n"
        "+-------------------------+------------+\n"
        "|line-too-long            |1           |\n"
        "+-------------------------+------------+\n"
        "\n"
        "\n"
        "\n"
        "\n"
        "-----------------------------------\n"
        "Your code has been rated at 0.00/10\n"
        "\n"
        "direct output\n"
    )


def test_multi_reporter_independant_messages() -> None:
    """Messages should not be modified by multiple reporters"""

    check_message = "Not modified"

    class ReporterModify(BaseReporter):
        def handle_message(self, msg: Message) -> None:
            msg.msg = "Modified message"

        def writeln(self, string: str = "") -> None:
            pass

        def _display(self, layout: Section) -> None:
            pass

    class ReporterCheck(BaseReporter):
        def handle_message(self, msg: Message) -> None:
            assert (
                msg.msg == check_message
            ), "Message object should not be changed by other reporters."

        def writeln(self, string: str = "") -> None:
            pass

        def _display(self, layout: Section) -> None:
            pass

    multi_reporter = MultiReporter([ReporterModify(), ReporterCheck()], lambda: None)

    message = Message(
        symbol="missing-docstring",
        msg_id="C0123",
        location=MessageLocationTuple("abspath", "path", "module", "obj", 1, 2, 1, 3),
        msg=check_message,
        confidence=HIGH,
    )

    multi_reporter.handle_message(message)

    assert (
        message.msg == check_message
    ), "Message object should not be changed by reporters."


def test_display_results_is_renamed() -> None:
    class CustomReporter(TextReporter):
        def _display(self, layout: Section) -> None:
            return None

    reporter = CustomReporter()
    with pytest.raises(AttributeError) as exc:
        # pylint: disable=no-member
        reporter.display_results()  # type: ignore[attr-defined]
    assert "no attribute 'display_results'" in str(exc)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

# pylint: disable=redefined-outer-name

from __future__ import annotations

import shutil
from collections.abc import Callable
from pathlib import Path

import pytest

from pylint.constants import IS_PYPY, PY39_PLUS
from pylint.testutils import FunctionalTestFile, LintModuleTest
from pylint.testutils.functional import LintModuleOutputUpdate

FIXTURE_DIRECTORY = Path(__file__).parent / "data/functional"
DIRECTORIES = list(FIXTURE_DIRECTORY.iterdir())


@pytest.fixture()
def lint_module_fixture(
    tmp_path: Path,
) -> Callable[[str], tuple[Path, Path, LintModuleOutputUpdate]]:
    def inner(base: str) -> tuple[Path, Path, LintModuleOutputUpdate]:
        filename = tmp_path / f"{base}.py"
        expected_output_file = tmp_path / f"{base}.txt"
        lmou = LintModuleOutputUpdate(
            test_file=FunctionalTestFile(str(tmp_path), str(filename))
        )
        return filename, expected_output_file, lmou

    return inner


def test_lint_module_output_update_fail_before(
    lint_module_fixture: Callable[[str], tuple[Path, Path, LintModuleOutputUpdate]]
) -> None:
    """There is a fail before the output need to be updated."""
    filename, expected_output_file, lmou = lint_module_fixture("foo")
    filename.write_text("", encoding="utf8")
    assert not expected_output_file.exists()
    with pytest.raises(AssertionError, match="1: disallowed-name"):
        lmou.runTest()
    assert not expected_output_file.exists()


def test_lint_module_output_update_effective(
    lint_module_fixture: Callable[[str], tuple[Path, Path, LintModuleOutputUpdate]]
) -> None:
    """The file is updated following a successful tests with wrong output."""
    filename, expected_output_file, lmou = lint_module_fixture("foo")
    filename.write_text("# [disallowed-name]\n", encoding="utf8")
    lmou.runTest()
    assert (expected_output_file).exists()
    assert (
        expected_output_file.read_text(encoding="utf8")
        == 'disallowed-name:1:0:None:None::"Disallowed name ""foo""":HIGH\n'
    )


def test_lint_module_output_update_remove_useless_txt(
    lint_module_fixture: Callable[[str], tuple[Path, Path, LintModuleOutputUpdate]]
) -> None:
    """The file is updated following a successful tests with wrong output."""
    filename, expected_output_file, lmou = lint_module_fixture("fine_name")
    expected_output_file.write_text("", encoding="utf8")
    filename.write_text("", encoding="utf8")
    lmou.runTest()
    assert not expected_output_file.exists()


@pytest.mark.skipif(
    IS_PYPY and not PY39_PLUS,
    reason="Requires accurate 'end_col' value to update output",
)
@pytest.mark.parametrize(
    "directory_path", DIRECTORIES, ids=[str(p) for p in DIRECTORIES]
)
def test_update_of_functional_output(directory_path: Path, tmp_path: Path) -> None:
    """Functional test for the functional tests' helper."""

    def _check_expected_output(_ftf: FunctionalTestFile) -> None:
        new_output_path = _ftf.expected_output
        assert Path(
            new_output_path
        ).exists(), "The expected output file does not exists"
        with open(new_output_path, encoding="utf8") as f:
            new_output = f.read()
        assert (
            new_output == "exec-used:7:0:7:14::Use of exec:UNDEFINED\n"
        ), f"The content was wrongly updated in {new_output_path}"

    def _assert_behavior_is_correct(
        _ftf: FunctionalTestFile,
        _lint_module: LintModuleTest,
        _lint_module_output_update: LintModuleOutputUpdate,
        _new_path: Path,
    ) -> None:
        new_path_str = str(_new_path)
        if "wrong_test" in new_path_str:
            expected = r'Wrong message\(s\) raised for "exec_used.py"'
            with pytest.raises(AssertionError, match=expected):
                _lint_module.runTest()
            # When the tests are wrong we do not update the output at all
            # and the test should fail
            with pytest.raises(AssertionError, match=expected):
                _lint_module_output_update.runTest()
        elif "ok_test" in new_path_str:
            if any(f"{x}_output" in new_path_str for x in ("wrong", "no", "broken")):
                with pytest.raises(
                    AssertionError, match='Wrong output for "exec_used.txt"'
                ):
                    _lint_module.runTest()
            elif "ok_output" in new_path_str:
                _lint_module.runTest()
                _check_expected_output(_ftf)
            else:
                raise AssertionError(f"Unhandled test case: {new_path_str}")

            # When the tests are ok we update the output whatever it's state
            # was originally
            _lint_module_output_update.runTest()
            _check_expected_output(_ftf)
        else:
            raise AssertionError(
                f"Do not pollute '{FIXTURE_DIRECTORY}' with unrelated "
                f"or badly named test files."
            )

    new_path = tmp_path / directory_path.name
    shutil.copytree(directory_path, new_path)
    for filename in new_path.iterdir():
        if filename.suffix != ".py":
            continue
        ftf = FunctionalTestFile(directory=str(new_path), filename=filename.name)
        # Standard functional test helper
        lint_module = LintModuleTest(ftf)
        # Functional test helper that automatically update the output
        lint_module_output_update = LintModuleOutputUpdate(ftf)

        _assert_behavior_is_correct(
            ftf, lint_module, lint_module_output_update, new_path
        )



"""Functional test"""
# pylint: disable=missing-function-docstring, invalid-name

def any_even(items):
    """Return True if the list contains any even numbers"""
    for item in items: # [consider-using-any-or-all]
        if item % 2 == 0:
            return True
    return False

def all_even(items):
    """Return True if the list contains all even numbers"""
    for item in items: # [consider-using-any-or-all]
        if not item % 2 == 0:
            return False
    return True

def any_uneven(items):
    """Return True if the list contains any uneven numbers"""
    for item in items: # [consider-using-any-or-all]
        if not item % 2 == 0:
            return True
    return False

def all_uneven(items):
    """Return True if the list contains all uneven numbers"""
    for item in items: # [consider-using-any-or-all]
        if item % 2 == 0:
            return False
    return True

def is_from_string(item):
    """Return True if one of parents of item is a string"""
    for parent in item.parents(): # [consider-using-any-or-all]
        if isinstance(parent, str):
            return True
    return False

def is_not_from_string(item):
    """Return True if one of parents of item isn't a string"""
    for parent in item.parents(): # [consider-using-any-or-all]
        if not isinstance(parent, str):
            return True
    return False

def nested_check(items):
    """Tests that for loops at deeper levels are picked up"""
    if items and len(items) > 5:
        print(items)
        for item in items: # [consider-using-any-or-all]
            if item in (1, 2, 3):
                return False
        return True
    print(items)
    return items[3] > 5

def words_contains_word(words):
    """Return whether words contains 'word'"""
    for word in words: # [consider-using-any-or-all]
        if word == "word":
            return True
    return False

def complicated_condition_check(items):
    """Case where we expect not any statement with a more complicated condition"""
    for item in items: # [consider-using-any-or-all]
        if item % 2 == 0 and (item % 3 == 0 or item > 15):
            return False
    return True

def is_from_decorator1(node):
    """Case where we expect a particularly long message to be emitted."""
    for ancestor in node: # [consider-using-any-or-all]
        if (
            ancestor.name in ("Exception", "BaseException")
            and ancestor.root().name == "Exception"
        ):
            return True
    return False

def is_from_decorator2(items):
    """Case where we expect an all statement because of negation in the condition"""
    for item in items: # [consider-using-any-or-all]
        if not(item % 2 == 0 and (item % 3 == 0 or item > 15)):
            return False
    return True

def is_from_decorator3(node):
    """Case where we expect a not all statement because of negation in the condition"""
    for ancestor in node: # [consider-using-any-or-all]
        if not (
            ancestor.name in ("Exception", "BaseException")
            and ancestor.root().name == "Exception"
        ):
            return True
    return False

def no_suggestion_if_not_if():
    """Do not emit if the for loop does not have the pattern we are looking for"""
    for val in range(1):
        var = val
        return var

def no_suggestion_if_not_bool(item):
    """Do not emit if the if-statement does not return a bool"""
    for parent in item.parents():
        if isinstance(parent, str):
            return "True"
    return "False"

def print_items(items):
    """Do not emit if there is no If condition in the for loop."""
    for item in items:
        print(item)
    return True

def print_items2(items):
    """Do not emit if anything besides a boolean is returned."""
    for item in items:
        return item
    return True

def print_items3(items):
    """Do not emit if anything besides a boolean is returned."""
    for _ in items:
        return False
    return items

def print_items4(items):
    """Do not emit if there is more logic which can cause side effects
    or become less readable in a list comprehension.
    """
    for item in items:
        if isinstance(item, str):
            print(item)
            return False
    return True

def is_from_decorator(node):
    """Do not emit if the if has an else condition. Generally implies more complicated logic."""
    for parent in node.node_ancestors():
        if isinstance(parent, str): # pylint: disable=no-else-return
            return True
        else:
            if parent in parent.selected_annotations:
                return False
            return False

def optimized_any_with_break(split_lines, max_chars):
    """False negative found in https://github.com/pylint-dev/pylint/pull/7697"""
    potential_line_length_warning = False
    for line in split_lines:  # [consider-using-any-or-all]
        if len(line) > max_chars:
            potential_line_length_warning = True
            break
    return potential_line_length_warning

def optimized_any_without_break(split_lines, max_chars):
    potential_line_length_warning = False
    for line in split_lines:  # [consider-using-any-or-all]
        if len(line) > max_chars:
            potential_line_length_warning = True
    return potential_line_length_warning

def print_line_without_break(split_lines, max_chars):
    potential_line_length_warning = False
    for line in split_lines:
        print(line)
        if len(line) > max_chars:
            potential_line_length_warning = True
    return potential_line_length_warning

def print_line_without_reassign(split_lines, max_chars):
    potential_line_length_warning = False
    for line in split_lines:
        if len(line) > max_chars:
            print(line)
    return potential_line_length_warning

def multiple_flags(split_lines, max_chars):
    potential_line_length_warning = False
    for line in split_lines:
        if len(line) > max_chars:
            num = 1
            print(num)
            potential_line_length_warning = True
    return potential_line_length_warning

s = ["hi", "hello", "goodbye", None]

flag = True
for i, elem in enumerate(s):
    if elem is None:
        continue
    cnt_s = cnt_t = 0
    for j in range(i, len(s)):
        if s[j] == elem:
            cnt_s += 1
            s[j] = None
            Flag = False

def with_elif(split_lines, max_chars):
    """
    Do not raise consider-using-any-or-all because the intent in this code
    is to iterate over all the lines (not short-circuit) and see what
    the last value would be.
    """
    last_longest_line = False
    for line in split_lines:
        if len(line) > max_chars:
            last_longest_line = True
        elif len(line) == max_chars:
            last_longest_line = False
    return last_longest_line

def first_even(items):
    """Return first even number"""
    for item in items:
        if item % 2 == 0:
            return item
    return None

def even(items):
    for item in items:
        if item % 2 == 0:
            return True
    return None

def iterate_leaves(leaves, current_node):
    results = []

    current_node.was_checked = True
    for leaf in leaves:
        if isinstance(leaf, bool):
            current_node.was_checked = False
        else:
            results.append(leaf)
    return results



"""Regression test for #5025"""

# pylint: disable=invalid-name, missing-docstring, too-few-public-methods


class AClass:  # [eq-without-hash]
    def __init__(self) -> None:
        self.x = 5

    def __eq__(self, other: object) -> bool:
        return isinstance(other, AClass) and other.x == self.x



# pylint: disable=invalid-name,missing-docstring,too-few-public-methods

# Slots with base that inherits from 'Generic'
# https://github.com/pylint-dev/pylint/issues/4509
# https://github.com/pylint-dev/astroid/issues/999

from typing import Generic, TypeVar
T = TypeVar("T")

class Base(Generic[T]):
    __slots__ = ()

class Foo(Base[T]):
    __slots__ = ['_value']

    def __init__(self, value: T):
        self._value = value
        self._bar = value  # [assigning-non-slot]



# pylint: disable=too-few-public-methods, import-error, missing-docstring, wrong-import-position
# pylint: disable=useless-super-delegation, unnecessary-pass


from typing import overload

from missing import Missing


class AAAA:
    """ancestor 1"""

    def __init__(self):
        print("init", self)


class BBBB:
    """ancestor 2"""

    def __init__(self):
        print("init", self)


class CCCC:
    """ancestor 3"""


class ZZZZ(AAAA, BBBB, CCCC):
    """derived class"""

    def __init__(self):  # [super-init-not-called]
        AAAA.__init__(self)


class NewStyleA:
    """new style class"""

    def __init__(self):
        super().__init__()
        print("init", self)


class NewStyleB(NewStyleA):
    """derived new style class"""

    def __init__(self):
        super().__init__()


class NewStyleC:
    """__init__ defined by assignment."""

    def xx_init(self):
        """Initializer."""
        pass

    __init__ = xx_init


class AssignedInit(NewStyleC):
    """No init called, but abstract so that is fine."""

    def __init__(self):
        self.arg = 0


class UnknownBases(Missing):
    """No false positives if the bases aren't known."""


class Parent:
    def __init__(self, num: int):
        self.number = num


class Child(Parent):
    @overload
    def __init__(self, num: int):
        ...

    @overload
    def __init__(self, num: float):
        ...

    def __init__(self, num):
        super().__init__(round(num))


# https://github.com/pylint-dev/pylint/issues/7742
# Crash when parent class has a class attribute named `__init__`
class NoInitMethod:
    __init__ = 42


class ChildNoInitMethod(NoInitMethod):
    def __init__(self):
        ...



""" docstring for file nullable_pattern.py """
from typing import Optional

class NullablePatterns:
    def return_nullable_1(self) -> int | None:
        """ Nullable return type using the | operator as mentioned in PEP 604, see https://peps.python.org/pep-0604 """
        pass

    def return_nullable_2(self) -> Optional[int]:
        pass



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

import logging
from pathlib import Path

from pytest import LogCaptureFixture

from pylint.testutils.configuration_test import get_expected_output

HERE = Path(__file__).parent
USER_SPECIFIC_PATH = HERE.parent.parent
DATA_DIRECTORY = HERE / "data"


def test_get_expected_output(caplog: LogCaptureFixture) -> None:
    caplog.set_level(logging.INFO)
    exit_code, _ = get_expected_output(DATA_DIRECTORY / "t.toml", USER_SPECIFIC_PATH)
    assert "Too much .out files" in str(caplog.text)
    assert exit_code == -1
    exit_code, _ = get_expected_output(DATA_DIRECTORY / "u.toml", USER_SPECIFIC_PATH)
    assert exit_code == -1
    assert "Wrong format for .out file name" in str(caplog.text)
    exit_code, _ = get_expected_output(DATA_DIRECTORY / "v.toml", USER_SPECIFIC_PATH)
    assert exit_code == 0
    assert ".out file does not exists" in str(caplog.text)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Check format checker helper functions."""

import os
import tempfile
import tokenize

import astroid

from pylint import lint, reporters
from pylint.checkers.base.basic_checker import BasicChecker
from pylint.checkers.format import FormatChecker
from pylint.testutils import CheckerTestCase, MessageTest, _tokenize_str


class TestSuperfluousParentheses(CheckerTestCase):
    CHECKER_CLASS = FormatChecker

    def testCheckKeywordParensHandlesValidCases(self) -> None:
        cases = [
            "if foo:",
            "if foo():",
            "if (x and y) or z:",
            "assert foo()",
            "assert ()",
            "if (1, 2) in (3, 4):",
            "if (a or b) in c:",
            "return (x for x in x)",
            "if (x for x in x):",
            "for x in (x for x in x):",
            "not (foo or bar)",
            "not (foo or bar) and baz",
            "return [x for x in (3 if 1 else [4])]",
            "return (x for x in ((3, 4) if 2 > 1 else (5, 6)))",
        ]
        with self.assertNoMessages():
            for code in cases:
                self.checker._check_keyword_parentheses(_tokenize_str(code), 0)

    def testCheckKeywordParensHandlesUnnecessaryParens(self) -> None:
        cases = [
            (MessageTest("superfluous-parens", line=1, args="if"), "if (foo):", 0),
            (
                MessageTest("superfluous-parens", line=1, args="if"),
                "if ((foo, bar)):",
                0,
            ),
            (
                MessageTest("superfluous-parens", line=1, args="if"),
                "if (foo(bar)):",
                0,
            ),
            (MessageTest("superfluous-parens", line=1, args="not"), "not (foo)", 0),
            (
                MessageTest("superfluous-parens", line=1, args="not"),
                "if not (foo):",
                1,
            ),
            (
                MessageTest("superfluous-parens", line=1, args="if"),
                "if (not (foo)):",
                0,
            ),
            (
                MessageTest("superfluous-parens", line=1, args="not"),
                "if (not (foo)):",
                2,
            ),
            (
                MessageTest("superfluous-parens", line=1, args="for"),
                "for (x) in (1, 2, 3):",
                0,
            ),
            (
                MessageTest("superfluous-parens", line=1, args="if"),
                "if (1) in (1, 2, 3):",
                0,
            ),
        ]
        for msg, code, offset in cases:
            with self.assertAddsMessages(msg):
                self.checker._check_keyword_parentheses(_tokenize_str(code), offset)

    def testNoSuperfluousParensWalrusOperatorIf(self) -> None:
        """Parenthesis change the meaning of assignment in the walrus operator
        and so are not always superfluous:
        """
        cases = [
            ("if (odd := is_odd(i))\n"),
            ("not (foo := 5)\n"),
        ]
        for code in cases:
            with self.assertNoMessages():
                self.checker.process_tokens(_tokenize_str(code))

    def testPositiveSuperfluousParensWalrusOperatorIf(self) -> None:
        """Test positive superfluous parens cases with the walrus operator."""
        cases = [
            (
                MessageTest("superfluous-parens", line=1, args="if"),
                "if ((x := y)):\n",
            ),
            (
                MessageTest("superfluous-parens", line=1, args="not"),
                "if not ((x := y)):\n",
            ),
        ]
        for msg, code in cases:
            with self.assertAddsMessages(msg):
                self.checker.process_tokens(_tokenize_str(code))

    def testCheckIfArgsAreNotUnicode(self) -> None:
        cases = [("if (foo):", 0), ("assert (1 == 1)", 0)]

        for code, offset in cases:
            self.checker._check_keyword_parentheses(_tokenize_str(code), offset)
            got = self.linter.release_messages()
            assert isinstance(got[-1].args, str)

    def testFuturePrintStatementWithoutParensWarning(self) -> None:
        code = """from __future__ import print_function
print('Hello world!')
"""
        tree = astroid.parse(code)
        with self.assertNoMessages():
            self.checker.process_module(tree)
            self.checker.process_tokens(_tokenize_str(code))

    def testKeywordParensFalsePositive(self) -> None:
        code = "if 'bar' in (DICT or {}):"
        with self.assertNoMessages():
            self.checker._check_keyword_parentheses(_tokenize_str(code), start=2)


class TestCheckSpace(CheckerTestCase):
    CHECKER_CLASS = FormatChecker

    def test_encoding_token(self) -> None:
        """Make sure the encoding token doesn't change the checker's behavior.

        _tokenize_str doesn't produce an encoding token, but
        reading a file does
        """
        with self.assertNoMessages():
            encoding_token = tokenize.TokenInfo(
                tokenize.ENCODING, "utf-8", (0, 0), (0, 0), ""
            )
            tokens = [
                encoding_token,
                *_tokenize_str("if (\n        None):\n    pass\n"),
            ]
            self.checker.process_tokens(tokens)


def test_disable_global_option_end_of_line() -> None:
    """Test for issue with disabling tokenizer messages
    that extend beyond the scope of the ast tokens
    """
    file_ = tempfile.NamedTemporaryFile("w", delete=False)
    with file_:
        file_.write(
            """
1
    """
        )
    # pylint: disable = too-many-try-statements
    try:
        linter = lint.PyLinter()
        checker = BasicChecker(linter)
        linter.register_checker(checker)
        args = linter._arguments_manager._parse_command_line_configuration(
            [file_.name, "-d", "pointless-statement"]
        )
        myreporter = reporters.CollectingReporter()
        linter.set_reporter(myreporter)
        linter.check(args)
        assert not myreporter.messages
    finally:
        os.remove(file_.name)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

__all__ = ["PackageToLint", "PRIMER_DIRECTORY_PATH"]

from pylint.testutils._primer.package_to_lint import (
    PRIMER_DIRECTORY_PATH,
    PackageToLint,
)



"""Tests for consider-using-augmented-assign."""

# pylint: disable=invalid-name,too-few-public-methods,import-error,consider-using-f-string,missing-docstring

from unknown import Unknown, elixir, random_object

x = 1

# summation is commutative (for integer and float, but not for string)
x = x + 3  # [consider-using-augmented-assign]
x = 3 + x  # [consider-using-augmented-assign]
x = x + "3"  # [consider-using-augmented-assign]
x = "3" + x

# We don't warn on intricate expressions as we lack knowledge of simplifying such
# expressions which is necessary to see if they can become augmented
x, y = 1 + x, 2 + x
x = 1 + x - 2
x = 1 + x + 2

# For anything other than a float or an int we only want to warn on
# assignments where the 'itself' is on the left side of the assignment
my_list = [2, 3, 4]
my_list = [1] + my_list


class MyClass:
    """Simple base class."""

    def __init__(self) -> None:
        self.x = 1
        self.x = self.x + 1  # [consider-using-augmented-assign]
        self.x = 1 + self.x  # [consider-using-augmented-assign]

        x = 1  # [redefined-outer-name]
        self.x = x


instance = MyClass()

x = instance.x + 1

my_str = ""
my_str = my_str + "foo"  # [consider-using-augmented-assign]
my_str = "foo" + my_str

my_bytes = b""
my_bytes = my_bytes + b"foo"  # [consider-using-augmented-assign]
my_bytes = b"foo" + my_bytes


def return_str() -> str:
    """Return a string."""
    return ""


# Currently we disregard all calls
my_str = return_str() + my_str
my_str = my_str % return_str()
my_str = my_str % 1  # [consider-using-augmented-assign]
my_str = my_str % (1, 2)  # [consider-using-augmented-assign]
my_str = "%s" % my_str
my_str = return_str() % my_str
my_str = Unknown % my_str
my_str = my_str % Unknown  # [consider-using-augmented-assign]

# subtraction is anti-commutative
x = x - 3  # [consider-using-augmented-assign]
x = 3 - x

# multiplication is commutative
x = x * 3  # [consider-using-augmented-assign]
x = 3 * x  # [consider-using-augmented-assign]

# division is not commutative
x = x / 3  # [consider-using-augmented-assign]
x = 3 / x

# integer division is not commutative
x = x // 3  # [consider-using-augmented-assign]
x = 3 // x

# Left shift operator is not commutative
x = x << 3  # [consider-using-augmented-assign]
x = 3 << x

# Right shift operator is not commutative
x = x >> 3  # [consider-using-augmented-assign]
x = 3 >> x

# modulo is not commutative
x = x % 3  # [consider-using-augmented-assign]
x = 3 % x

# exponential is not commutative
x = x**3  # [consider-using-augmented-assign]
x = 3**x

# XOR is commutative
x = x ^ 3  # [consider-using-augmented-assign]
x = 3 ^ x  # [consider-using-augmented-assign]

# Bitwise AND operator is commutative
x = x & 3  # [consider-using-augmented-assign]
x = 3 & x  # [consider-using-augmented-assign]

# Bitwise OR operator is commutative
x = x | 3  # [consider-using-augmented-assign]
x = 3 | x  # [consider-using-augmented-assign]

x = x > 3
x = 3 > x

x = x < 3
x = 3 < x

x = x >= 3
x = 3 >= x

x = x <= 3
x = 3 <= x

# Should also apply to dictionaries when subscripts are the same
my_dict = {"count": 5}
my_dict["count"] = my_dict["count"] + 2  # [consider-using-augmented-assign]
my_dict["apples"] = my_dict["count"] + 1

my_dict = {"msg": {"title": "Hello"}}
my_dict["msg"]["title"] = my_dict["msg"]["title"] + " world!"  # [consider-using-augmented-assign]
my_dict["msg"]["body"] = my_dict["msg"]["title"] + " everyone, this should not raise messages"

# Applies to lists as well
test_list = [1, 2]
test_list[0] = test_list[0] - 1  # [consider-using-augmented-assign]
test_list[1] = test_list[0] + 10

# Can't infer, don't mark message
random_object[elixir] = random_object[elixir] + 1

# https://github.com/pylint-dev/pylint/issues/8086
# consider-using-augmented-assign should only be flagged
# if names attribute names match exactly.

class A:
    def __init__(self) -> None:
        self.a = 1
        self.b = A()
        self.c = [1, 2, 3]

    def test(self) -> None:
        self.a = self.a + 1  # [consider-using-augmented-assign]
        self.b.a = self.a + 1  # Names don't match!

    def line(self) -> None:
        self.c[1] = self.c[1] + 1  # [consider-using-augmented-assign]



# pylint: disable=missing-docstring,invalid-name
# https://github.com/pylint-dev/pylint/issues/3535

import enum

class A(enum.Enum):
    pass

class B(A):
    x = enum.auto()

print(B.__members__['x'])



""" Regression test for https://github.com/pylint-dev/pylint/issues/3405. """

import dataclasses
from typing import ClassVar


@dataclasses.dataclass
class Foo:
    """ClassVar attribute should be matched against class-attribute-rgx, not attr-rgx"""
    # class-attribute-rgx='^y$'
    x: ClassVar[int] = 0  # [invalid-name]



"""bla"""


from ..n.non import non_init_parent_called


class Aaaa(non_init_parent_called.AAAA):  # [too-few-public-methods]
    """test dotted name in ancestors"""
    def __init__(self):
        non_init_parent_called.AAAA.__init__(self)



"""Tests for missing-function-docstring"""
# pylint: disable=unused-argument, missing-class-docstring, too-few-public-methods, unused-variable


def func(tion):  # [missing-function-docstring]
    pass


def func_two(tion):
    """Documented"""

    def inner(fun):
        # Not documented
        pass


class MyClass:
    def __init__(self, my_param: int) -> None:  # [missing-function-docstring]
        pass



"""Tests for missing-return-doc and missing-return-type-doc with accept-no-returns-doc = no"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument


def warns_no_docstring(self):  # [missing-return-doc, missing-return-type-doc]
    return False


# this function doesn't require a docstring, because its name starts
# with an '_' (no-docstring-rgx):
def _function(some_arg: int) -> int:
    _ = some_arg
    return 0



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

# pylint: disable=redefined-outer-name

from __future__ import annotations

import os
from collections.abc import Callable
from pathlib import Path

import pytest

from pylint import checkers
from pylint.checkers import BaseChecker
from pylint.lint import PyLinter
from pylint.lint.run import _cpu_count
from pylint.reporters import BaseReporter
from pylint.testutils import MinimalTestReporter

HERE = Path(__file__).parent


@pytest.fixture()
def tests_directory() -> Path:
    return HERE


@pytest.fixture
def linter(
    checker: type[BaseChecker] | None,
    register: Callable[[PyLinter], None] | None,
    enable: str | None,
    disable: str | None,
    reporter: type[BaseReporter],
) -> PyLinter:
    _linter = PyLinter()
    _linter.set_reporter(reporter())
    checkers.initialize(_linter)
    if register:
        register(_linter)
    if checker:
        _linter.register_checker(checker(_linter))
    if disable:
        for msg in disable:
            _linter.disable(msg)
    if enable:
        for msg in enable:
            _linter.enable(msg)
    os.environ.pop("PYLINTRC", None)
    return _linter


@pytest.fixture(scope="module")
def checker() -> None:
    return None


@pytest.fixture(scope="module")
def register() -> None:
    return None


@pytest.fixture(scope="module")
def enable() -> None:
    return None


@pytest.fixture(scope="module")
def disable() -> None:
    return None


@pytest.fixture(scope="module")
def reporter() -> type[MinimalTestReporter]:
    return MinimalTestReporter


def pytest_addoption(parser: pytest.Parser) -> None:
    parser.addoption(
        "--primer-stdlib",
        action="store_true",
        default=False,
        help="Run primer stdlib tests",
    )
    parser.addoption(
        "--minimal-messages-config",
        action="store_true",
        default=False,
        help=(
            "Disable all messages that are not explicitly expected when running functional tests. "
            "This is useful for finding problems with the @only_required_for_messages / @check_messages "
            "decorator, but can also produce false negatives if a functional test file only tests for "
            "false positive of messages and thus does not declare which messages are expected."
        ),
    )


def pytest_collection_modifyitems(
    config: pytest.Config, items: list[pytest.Function]
) -> None:
    """Convert command line options to markers."""
    # Add skip_primer_stdlib mark
    if not config.getoption("--primer-stdlib"):
        skip_primer_stdlib = pytest.mark.skip(
            reason="need --primer-stdlib option to run"
        )
        for item in items:
            if "primer_stdlib" in item.keywords:
                item.add_marker(skip_primer_stdlib)

    # Add skip_cpu_cores mark
    if _cpu_count() < 2:
        skip_cpu_cores = pytest.mark.skip(
            reason="Need 2 or more cores for test to be meaningful"
        )
        for item in items:
            if "needs_two_cores" in item.keywords:
                item.add_marker(skip_cpu_cores)



r"""A raw docstring.
"""


def look_busy():
    xxxx = 1
    yyyy = 2
    zzzz = 3
    wwww = 4
    vvvv = xxxx + yyyy + zzzz + wwww
    return vvvv


def look_busy_two():
    xxxx = 1
    yyyy = 2
    zzzz = 3
    wwww = 4
    vvvv = xxxx + yyyy + zzzz + wwww
    return vvvv



"""Tests for missing-return-doc and missing-return-type-doc for Numpy style docstrings
with accept-no-returns-doc = no"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument, too-few-public-methods


def my_func(self, doc_type):  # [missing-return-doc]
    """warn_partial_numpy_returns_type

    Arguments
    ---------
    doc_type : str
        Numpy

    Returns
    -------
    bool
    """
    return False


def my_func(self, doc_type):  # [missing-return-doc, missing-return-type-doc]
    """warn_missing_numpy_returns

    Arguments
    ---------
    doc_type : str
        Numpy
    """
    return False


def my_func(self):  # [missing-return-doc]
    """warns_numpy_return_list_of_custom_class_without_description

    Returns
    -------
        list(:class:`mymodule.Class`)
    """
    return [mymodule.Class()]


class Foo:
    """test_finds_missing_property_return_type_numpy
    Example of a property having return documentation in
    a numpy style docstring
    """

    @property
    def foo_prop(self):  # [missing-return-type-doc]
        """docstring ...

        Raises
        ------
        RuntimeError
            Always
        """
        raise RuntimeError()
        return 10  # [unreachable]


class Foo:
    """test_ignores_non_property_return_type_numpy
    Example of a class function trying to use `type` as return
    documentation in a numpy style docstring
    """

    def foo_method(self):  # [missing-return-doc, missing-return-type-doc]
        """int: docstring ...

        Raises
        ------
        RuntimeError
            Always
        """
        print(self)
        raise RuntimeError()
        return 10  # [unreachable]


class Foo:
    """test_non_property_annotation_return_type_numpy
    Example of a class function trying to use `type` as return
    documentation in a numpy style docstring
    """

    def foo_method(self) -> int:  # [missing-return-doc]
        """int: docstring ...

        Raises
        ------
        RuntimeError
            Always
        """
        print(self)
        raise RuntimeError()
        return 10  # [unreachable]



"""Warnings for using open() without specifying an encoding"""
# pylint: disable=consider-using-with, too-few-public-methods
import dataclasses
import io
import locale
from pathlib import Path
from typing import Optional

FILENAME = "foo.bar"
open(FILENAME, "w", encoding="utf-8")
open(FILENAME, "wb")
open(FILENAME, "w+b")
open(FILENAME)  # [unspecified-encoding]
open(FILENAME, "wt")  # [unspecified-encoding]
open(FILENAME, "w+")  # [unspecified-encoding]
open(FILENAME, "w", encoding=None)  # [unspecified-encoding]
open(FILENAME, "r")  # [unspecified-encoding]

with open(FILENAME, encoding="utf8", errors="surrogateescape") as f:
    pass

LOCALE_ENCODING = locale.getlocale()[1]
with open(FILENAME, encoding=LOCALE_ENCODING) as f:
    pass

with open(FILENAME) as f:  # [unspecified-encoding]
    pass

with open(FILENAME, encoding=None) as f:  # [unspecified-encoding]
    pass

LOCALE_ENCODING = None
with open(FILENAME, encoding=LOCALE_ENCODING) as f:  # [unspecified-encoding]
    pass

io.open(FILENAME, "w+b")
io.open_code(FILENAME)
io.open(FILENAME)  # [unspecified-encoding]
io.open(FILENAME, "wt")  # [unspecified-encoding]
io.open(FILENAME, "w+")  # [unspecified-encoding]
io.open(FILENAME, "w", encoding=None)  # [unspecified-encoding]

with io.open(FILENAME, encoding="utf8", errors="surrogateescape") as f:
    pass

LOCALE_ENCODING = locale.getlocale()[1]
with io.open(FILENAME, encoding=LOCALE_ENCODING) as f:
    pass

with io.open(FILENAME) as f:  # [unspecified-encoding]
    pass

with io.open(FILENAME, encoding=None) as f:  # [unspecified-encoding]
    pass

LOCALE_ENCODING = None
with io.open(FILENAME, encoding=LOCALE_ENCODING) as f:  # [unspecified-encoding]
    pass

LOCALE_ENCODING = locale.getlocale()[1]
Path(FILENAME).read_text(encoding=LOCALE_ENCODING)
Path(FILENAME).read_text(encoding="utf8")
Path(FILENAME).read_text("utf8")

LOCALE_ENCODING = None
Path(FILENAME).read_text()  # [unspecified-encoding]
Path(FILENAME).read_text(encoding=None)  # [unspecified-encoding]
Path(FILENAME).read_text(encoding=LOCALE_ENCODING)  # [unspecified-encoding]

LOCALE_ENCODING = locale.getlocale()[1]
Path(FILENAME).write_text("string", encoding=LOCALE_ENCODING)
Path(FILENAME).write_text("string", encoding="utf8")

LOCALE_ENCODING = None
Path(FILENAME).write_text("string")  # [unspecified-encoding]
Path(FILENAME).write_text("string", encoding=None)  # [unspecified-encoding]
Path(FILENAME).write_text("string", encoding=LOCALE_ENCODING)  # [unspecified-encoding]

LOCALE_ENCODING = locale.getlocale()[1]
Path(FILENAME).open("w+b")
Path(FILENAME).open()  # [unspecified-encoding]
Path(FILENAME).open("wt")  # [unspecified-encoding]
Path(FILENAME).open("w+")  # [unspecified-encoding]
Path(FILENAME).open("w", encoding=None)  # [unspecified-encoding]
Path(FILENAME).open("w", encoding=LOCALE_ENCODING)


# Tests for storing data about open calls.
# Most of these are regression tests for a crash
# reported in https://github.com/pylint-dev/pylint/issues/5321

# -- Constants
MODE = "wb"
open(FILENAME, mode=MODE)


# -- Functions
def return_mode_function():
    """Return a mode for open call"""
    return "wb"

open(FILENAME, mode=return_mode_function())


# -- Classes
class IOData:
    """Class that returns mode strings"""

    mode = "wb"

    def __init__(self):
        self.my_mode = "wb"

    @staticmethod
    def my_mode_method():
        """Returns a pre-defined mode"""
        return "wb"

    @staticmethod
    def my_mode_method_returner(mode: str) -> str:
        """Returns the supplied mode"""
        return mode


open(FILENAME, mode=IOData.mode)
open(FILENAME, mode=IOData().my_mode)
open(FILENAME, mode=IOData().my_mode_method())
open(FILENAME, mode=IOData().my_mode_method_returner("wb"))
# Invalid value but shouldn't crash, reported in https://github.com/pylint-dev/pylint/issues/5321
open(FILENAME, mode=IOData)


# -- Dataclasses
@dataclasses.dataclass
class IOArgs:
    """Dataclass storing information about how to open a file"""

    encoding: Optional[str]
    mode: str


args_good_one = IOArgs(encoding=None, mode="wb")

# Test for crash reported in https://github.com/pylint-dev/pylint/issues/5321
open(FILENAME, args_good_one.mode, encoding=args_good_one.encoding)

# Positional arguments
open(FILENAME, "w", -1, "utf-8")
open(FILENAME, "w", -1)  # [unspecified-encoding]

Path(FILENAME).open("w", -1, "utf-8")
Path(FILENAME).open("w", -1)  # [unspecified-encoding]

Path(FILENAME).read_text("utf-8")
Path(FILENAME).read_text()  # [unspecified-encoding]

Path(FILENAME).write_text("string", "utf-8")
Path(FILENAME).write_text("string")  # [unspecified-encoding]

# Test for crash reported in https://github.com/pylint-dev/pylint/issues/5731
open(FILENAME, mode=None)  # [bad-open-mode, unspecified-encoding]

# Test for crash reported in https://github.com/pylint-dev/pylint/issues/6414
open('foo', mode=2)  # [bad-open-mode, unspecified-encoding]

# Infer kwargs
KWARGS = {"mode": "rb"}
open(FILENAME, **KWARGS)

KWARGS = {"mode": "w", "encoding": "utf-8"}
open(FILENAME, **KWARGS)
io.open(FILENAME, **KWARGS)
Path(FILENAME).open(**KWARGS)

KWARGS = {"mode": 5}
open(FILENAME, **KWARGS)  # [bad-open-mode, unspecified-encoding]
io.open(FILENAME, **KWARGS)  # [bad-open-mode, unspecified-encoding]

KWARGS = {"mode": "wt", "encoding": None}
with open(FILENAME, **KWARGS) as fd:  # [unspecified-encoding]
    pass

Path(FILENAME).open(**KWARGS)  # [unspecified-encoding]

KWARGS = {"encoding": None}
Path(FILENAME).write_text("hello", **KWARGS)  # [unspecified-encoding]

KWARGS = {"encoding": "utf-8"}
Path(FILENAME).write_text("goodbye", **KWARGS)

# No one does it this way, but it is a possibility
KWARGS = {"encoding": None}
Path(FILENAME).read_text(**KWARGS)  # [unspecified-encoding]

KWARGS = {"encoding": "utf-8"}
Path(FILENAME).read_text(**KWARGS)



"""Check invalid value returned by __repr__ """

# pylint: disable=too-few-public-methods,missing-docstring,import-error,unnecessary-lambda-assignment
import six

from missing import Missing


class FirstGoodRepr:
    """__repr__ returns <type 'str'>"""

    def __repr__(self):
        return "some repr"


class SecondGoodRepr:
    """__repr__ returns <type 'str'>"""

    def __repr__(self):
        return str(123)


class ReprMetaclass(type):
    def __repr__(cls):
        return "some repr"


@six.add_metaclass(ReprMetaclass)
class ThirdGoodRepr:
    """Repr through the metaclass."""


class FirstBadRepr:
    """ __repr__ returns bytes """

    def __repr__(self):  # [invalid-repr-returned]
        return b"123"


class SecondBadRepr:
    """ __repr__ returns int """

    def __repr__(self):  # [invalid-repr-returned]
        return 1


class ThirdBadRepr:
    """ __repr__ returns node which does not have 'value' in AST """

    def __repr__(self):  # [invalid-repr-returned]
        return lambda: "some repr"


class AmbiguousRepr:
    """ Uninferable return value """

    __repr__ = lambda self: Missing


class AnotherAmbiguousRepr:
    """Potential uninferable return value"""

    def __repr__(self):
        return str(Missing)



"""
Regression test for `no-member`.
See: https://github.com/pylint-dev/pylint/issues/2567
"""

# pylint: disable=missing-docstring,too-few-public-methods

import contextlib


@contextlib.contextmanager
def context_manager():
    try:
        yield
    finally:
        pass


cm = context_manager()
cm.__enter__()
cm.__exit__(None, None, None)


@contextlib.contextmanager
def other_context_manager():
    try:
        yield
    finally:
        pass


with other_context_manager():  # notice the function call
    pass



# pylint: disable=too-few-public-methods,missing-docstring
"""check method hiding ancestor attribute
"""
import something_else as functools  # pylint: disable=import-error


class Parent:
    def __init__(self):
        self._protected = None


class Child(Parent):
    @functools().cached_property
    def _protected(self):
        # This test case is only valid for python3.9 and above
        pass



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from pathlib import Path

import pytest

HERE = Path(__file__).parent


@pytest.fixture()
def file_to_lint_path() -> str:
    return str(HERE / "file_to_lint.py")



# pylint: disable=missing-docstring,invalid-name
from functools import cached_property


# https://github.com/pylint-dev/pylint/issues/4023
# False-positive 'invalid-overridden-method' with 'cached_property'
class Parent:
    @property
    def value(self):
        return 42

    def func(self):
        return False


class Child(Parent):
    @cached_property
    def value(self):
        return 2**6

    @cached_property
    def func(self):  # [invalid-overridden-method]
        return 42



"""
Checks that iterable metaclasses are recognized by pylint.
"""
# pylint: disable=missing-docstring,too-few-public-methods,unused-argument,bad-mcs-method-argument
# pylint: disable=wrong-import-position
# metaclasses as iterables
class Meta(type):
    def __iter__(self):
        return iter((1, 2, 3))

class SomeClass(metaclass=Meta):
    pass


for i in SomeClass:
    print(i)
for i in SomeClass():  # [not-an-iterable]
    print(i)



"""
non ascii variable defined in a function

This test is only for 3.8 as the starting column is incorrect
"""


def okay(
    just_some_thing_long_again: str,
    lol_very_long_argument: str,
    łol: str,  # [non-ascii-name]
) -> bool:
    """Be okay, yeah?"""
    # Usage should not raise a second error
    print(just_some_thing_long_again, lol_very_long_argument, łol)
    return True


# Usage should raise a second error
okay(
    "A          VVVVVVVEEEERRRRRRRRRRYYYYYYYYYY    LONG TIME               ",
    lol_very_long_argument="a",
    łol="b",  # [non-ascii-name]
)



"""Tests for invalid-name checker with properties."""
# pylint: disable=missing-class-docstring, missing-function-docstring, deprecated-decorator
# pylint: disable=too-few-public-methods


import abc


def custom_prop(F): # [invalid-name]
    return property(F)


class FooClass:
    """Test properties with attr-rgx and property-classes options."""

    @property
    def FOO(self):
        pass

    @property
    def bar(self):  # [invalid-name]
        pass

    @abc.abstractproperty
    def BAZ(self):
        pass

    @custom_prop
    def QUX(self):
        pass


class AnotherFooClass:
    """Test property setter for pattern set in attr-rgx."""

    @property
    def foo(self): # [invalid-name]
        pass

    @foo.setter
    def FOOSETTER(self):
        pass



# pylint: disable=missing-docstring, invalid-name
from enum import Enum, Flag, IntEnum, auto


class Issue1932(IntEnum):
    """https://github.com/pylint-dev/pylint/issues/1932"""

    FOO = 1

    def whats_my_name(self):
        return self.name.lower()


class Issue2062(Enum):
    """https://github.com/pylint-dev/pylint/issues/2062"""

    FOO = 1
    BAR = 2

    def __str__(self):
        return self.name.lower()


class OrderedEnum(Enum):
    def __ge__(self, other):
        if self.__class__ is other.__class__:
            return self.value >= other.value  # line 11
        return NotImplemented

    def __gt__(self, other):
        if self.__class__ is other.__class__:
            return self.value > other.value  # line 16
        return NotImplemented


class Color(OrderedEnum):
    red = 0
    green = 1


class People(Enum):
    jack = 0
    john = 1


print(Color.red.value)  # line 29
print(People.jack.name)


class BaseEnum(Enum):
    def some_behavior(self):
        pass


class MyEnum(BaseEnum):

    FOO = 1
    BAR = 2


print(MyEnum.FOO.value)


class TestBase(Enum):
    """Adds a special method to enums."""

    def hello_pylint(self) -> str:
        """False positive."""
        return self.name


class TestEnum(TestBase):
    """Tests the false positive for enums."""

    a = auto()
    b = auto()


test_enum = TestEnum.a
assert test_enum.hello_pylint() == test_enum.name


# Check combinations of Flag members using the bitwise operators (&, |, ^, ~)
# https://github.com/pylint-dev/pylint/issues/7381
class Colour(Flag):
    NONE = 0
    RED = 2
    GREEN = 2
    BLUE = 4


and_expr = Colour.RED & Colour.GREEN & Colour.BLUE
and_expr_with_complement = ~Colour.RED & ~Colour.GREEN & ~Colour.BLUE
or_expr = Colour.RED | Colour.GREEN | Colour.BLUE
xor_expr = Colour.RED ^ Colour.GREEN ^ Colour.BLUE
